# -*- coding: UTF-8 -*-
from django.test import TestCase
from l10n.validators import aupostcode, capostcode, uspostcode
from l10n import l10n_settings
from l10n.utils import moneyfmt
from decimal import Decimal

class AUPostCodeTest(TestCase):
    def test_valid(self):
        code = aupostcode.validate("2000")
        self.assertEqual('2000', code)
        code = aupostcode.validate(" 2000 ")
        self.assertEqual('2000', code)
    
    def test_invalid(self):
        try:
            code = capostcode.validate("")
            self.fail('Invalid blank postal code not caught')
        except:
            pass
        
        try:
            code = capostcode.validate("no")
            self.fail('Invalid postal code "no" not caught')
        except:
            pass
        
class CAPostCodeTest(TestCase):
    def test_valid(self):
        code = capostcode.validate("M5V2T6")
        self.assertEqual('M5V2T6', code)
        code = capostcode.validate("m5v2t6")
        self.assertEqual('M5V2T6', code)
    
    def test_invalid(self):
        try:
            code = capostcode.validate("")
            self.fail('Invalid blank postal code not caught')
        except:
            pass
        
        try:
            code = capostcode.validate("no")
            self.fail('Invalid postal code "no" not caught')
        except:
            pass
        
        try:
            code = capostcode.validate("M5V M5V")
            self.fail('Invalid postal code "M5V M5V" not caught')
        except:
            pass
        
        try:
            code = capostcode.validate("D5V 2T6")
            self.fail('Invalid postal code "D5V 2T6" not caught -- "D" is not a valid major geographic area or province.')
        except:
            pass

class USPostCodeTest(TestCase):
    def test_five_digit(self):
        zipcode = uspostcode.validate("66044")
        self.assertEqual('66044', zipcode)
        try:
            zipcode = uspostcode.validate(" 66044 ")
            self.fail("Invalid postal code not caught")
        except:
            pass

    def test_nine_digit(self):
        zipcode = uspostcode.validate("94043-1351")
        self.assertEqual('94043-1351', zipcode)
        try:
            zipcode = uspostcode.validate(" 94043-1351 ")
            self.fail('Invalide postal code not caught')
        except:
            pass
    
    def test_invalid(self):
        try:
            code = uspostcode.validate("")
            self.fail('Invalid blank postal code not caught')
        except:
            pass
        
        try:
            zipcode = uspostcode.validate("no")
            self.fail('Invalid ZIP code "no" not caught')
        except:
            pass
            
class MoneyFmtTest(TestCase):
    
    def testUSD(self):
        l10n_settings.set_l10n_setting('default_currency', 'USD')
        
        val = Decimal('10.00')
        self.assertEqual(moneyfmt(val), '$10.00')

        self.assertEqual(moneyfmt(val, currency_code='USD'), '$10.00')


    def testGBP(self):
        l10n_settings.set_l10n_setting('default_currency', 'GBP')

        val = Decimal('10.00')
        self.assertEqual(moneyfmt(val), '£10.00')

        self.assertEqual(moneyfmt(val, currency_code='GBP'), '£10.00')
        self.assertEqual(moneyfmt(val, currency_code='USD'), '$10.00')
        
        val = Decimal('-100.00')
        self.assertEqual(moneyfmt(val), '-£100.00')

    def testFake(self):
        currencies = l10n_settings.get_l10n_setting('currency_formats')
        currencies['FAKE'] = {'symbol': '^ ', 'positive' : "%(val)0.2f ^", 'negative': "(%(val)0.2f) ^", 'decimal' : ','}
        
        l10n_settings.set_l10n_setting('currency_formats', currencies)
        
        val = Decimal('10.00')
        self.assertEqual(moneyfmt(val, currency_code='FAKE'), '10,00 ^')

        val = Decimal('-50.00')
        self.assertEqual(moneyfmt(val, currency_code='FAKE'), '(50,00) ^')


        
        


"""Tags for manipulating brands on templates."""

from django.template import Library, Node, TemplateSyntaxError
from satchmo_ext.brand.models import Brand
from satchmo_utils.templatetags import get_filter_args

register = Library()

class BrandListNode(Node):
    """Template Node tag which pushes the brand list into the context"""
    def __init__(self, var, nodelist):
        self.var = var
        self.nodelist = nodelist

    def render(self, context):
        brands = Brand.objects.active()
        context[self.var] = brands
        context.push()
        context[self.var] = brands
        output = self.nodelist.render(context)
        context.pop()
        return output            
            
def do_brandlistnode(parser, token):
    """Push the brand list into the context using the given variable name.

    Sample usage::

        {% brand_list as var %}
        
    """
    args = token.split_contents()
    if len(args) != 3:
        raise TemplateSyntaxError("%r tag expecting '[slug] as varname', got: %s" % (args[0], args))
    
    var = args[2]
    nodelist = parser.parse(('endbrand_list',))
    parser.delete_first_token()
    return BrandListNode(var, nodelist)

register.tag('brand_list', do_brandlistnode)



from django.core.exceptions import ImproperlyConfigured
from django.utils.functional import curry
from django.utils.datastructures import SortedDict

libraries = {}

def _register(lib, store, prefix, options, do_error, call):
    'generic module library registration decorator passthrough'
    if call is None:
        if do_error:
            raise RuntimeError, "No callable recieved."
        return curry(_register, lib, store, prefix, options, True)
    if not callable(call):
        raise SyntaxError, "must supply callable"
    if not hasattr(call, '__module__'):
        raise SyntaxError, "callable must have __module__ defined"
    if not hasattr(call, '__name__'):
        raise SyntaxError, "callable must have __name__ defined"
    name = call.__name__
    if prefix: name = prefix + '.' + call.__name__
    if name in store:
        raise RuntimeError, "library already has a call for " + name
    if lib.app_name is None:
        lib.app_name = call.__module__
        libraries[lib.app_name] = lib
    elif lib.app_name != call.__module__:
        raise RuntimeError, ("library is for module %s, not %s." %
                             (lib.app_name, call.__module__))
    store[name] = call
    call.options = options
    return call

class Library(object):
    def __init__(self):
        self.app_name = None
        self.point_calls = SortedDict()
        self.plugin_calls = SortedDict()

    @property
    def plugin_points(self):
        return ('.'.join([self.app_name, p]) for p in self.point_calls)

    @property
    def plugins(self):
        return ('.'.join([self.app_name, p]) for p in self.plugin_calls)

    def get_plugin_point_call(self, name):
        return self.point_calls[name]

    def get_plugin_call(self, name):
        return self.plugin_calls[name]

    def plugin_point(self, call=None, **options):
        return _register(self, self.point_calls, '', options, False, call)

    def plugin(self, call=None, **options):
        if not callable(call) and cal is not None:
            point_app = call
            call = None
        else:
            point_app = ''
        return _register(self, self.plugin_calls, point_app, options,
                         False, call)

def get_library(app):
    return libraries.get(app, None)



from django.test import TestCase
from satchmo_store.contact.forms import ContactInfoForm
from satchmo_store.contact.models import *
from l10n.models import Country
from satchmo_store.shop.models import Config
import datetime

class ContactTest(TestCase):
    fixtures = ['l10n-data.yaml', 'test-config.yaml']
    
    def setUp(self):
        self.US = Country.objects.get(iso2_code__iexact="US")
    
    def test_base_contact(self):
        """Test creating a contact"""

        contact1 = Contact.objects.create(first_name="Jim", last_name="Tester", 
            role=ContactRole.objects.get(pk='Customer'), email="Jim@JimWorld.com")
            
        self.assertEqual(contact1.full_name, u'Jim Tester')

        # Add a phone number for this person and make sure that it's the default
        phone1 = PhoneNumber.objects.create(contact=contact1, type='Home', phone="800-111-9900")
        self.assert_(contact1.primary_phone)
        self.assertEqual(contact1.primary_phone.phone, '800-111-9900')
        self.assertEqual(phone1.type, 'Home') 

        # Make sure that new primary phones become the default, and that
        # non-primary phones don't become the default when a default already exists.
        phone2 = PhoneNumber.objects.create(contact=contact1, type='Work', phone="800-222-9901", primary=True)
        phone3 = PhoneNumber.objects.create(contact=contact1, type="Mobile", phone="800-333-9902")
        self.assertEqual(contact1.primary_phone, phone2)

        #Add an address & make sure it is default billing and shipping
        add1 = AddressBook.objects.create(contact=contact1, description="Home Address",
            street1="56 Cool Lane", city="Niftyville", state="IA", postal_code="12344",
            country=self.US)
        self.assert_(contact1.billing_address)
        self.assertEqual(contact1.billing_address, add1)
        self.assertEqual(contact1.billing_address.description, "Home Address")
        self.assertEqual(add1.street1, "56 Cool Lane")

        self.assertEqual(contact1.billing_address, contact1.shipping_address)
        self.assertEqual(contact1.billing_address.street1, "56 Cool Lane")

        #Add a new shipping address
        add2 = AddressBook(description="Work Address", street1="56 Industry Way", city="Niftytown", 
            state="IA", postal_code="12366", country=self.US, is_default_shipping=True)
        add2.contact = contact1
        add2.save()
        contact1.save()
        self.assertNotEqual(contact1.billing_address, contact1.shipping_address)
        self.assertEqual(contact1.billing_address.description, "Home Address")
        self.assertEqual(contact1.shipping_address.description, "Work Address")
        
    def test_contact_org(self):
        contact1 = Contact.objects.create(first_name="Org", last_name="Tester", 
            role=ContactRole.objects.get(pk='Customer'), email="org@example.com")
        org = Organization.objects.by_name('The Testers', create=True)
        self.assert_(org)
        self.assertEqual(org.role.name, 'Customer')
        org2 = Organization.objects.by_name('The Testers', create=True)
        self.assertEqual(org, org2)
        
class ContactInfoFormTest(TestCase):
    fixtures = ['l10n-data.yaml', 'test_shop.yaml', 'test-config.yaml']
    
    def test_missing_first_and_last_name_should_not_raise_exception(self):
        shop = Config.objects.get_current()
        form = ContactInfoForm(shop=shop, contact=None, data={'phone':'800-111-9900'})
        self.assertEqual(False, form.is_valid())
        
class ContactInfoFormLANGUAGETest(TestCase):
    fixtures = ['l10n_data.xml', 'test_intl_shop.yaml', 'test-config.yaml']
    
    def test_company(self):
        contact = Contact.objects.create()
        data = {
            'email': 'company@satchmoproject.com', 'first_name': 'Test', 'last_name': 'Company','phone':'123-111-4411',

            'street1': "56 Cool Lane", 'city': "Niftyville", 'state': "IA", 'postal_code': "12344", 'country': 231,
            'ship_street1': "56 Industry Way", 'ship_city': "Niftytown", 'ship_state': "IA", 'ship_postal_code': "12366", 'ship_country': 231,
            'organization' : 'Testers Anonymous',
            }
        shop = Config.objects.get_current()
        form = ContactInfoForm(data=data, shop=shop, contact=contact)
        #print "ERR: %s" % form.errors
        self.assertEqual(True, form.is_valid())
        contactid = form.save(contact=contact)
        self.assertEqual(contact.id, contactid)
        self.assert_(contact.organization)
        self.assertEqual(contact.organization.name, 'Testers Anonymous')
    
    
    def test_country_specific_validation(self):
        shop = Config.objects.get_current()
        
        # US
        
        # a valid one
        contact = Contact.objects.create()
        data = {
            'email': 'test_email@satchmoproject.com', 'first_name': 'Test', 'last_name': 'McTestalot','phone':'123-111-4411',

            'street1': "56 Cool Lane", 'city': "Niftyville", 'state': "IA", 'postal_code': "12344", 'country': 231,
            'ship_street1': "56 Industry Way", 'ship_city': "Niftytown", 'ship_state': "IA", 'ship_postal_code': "12366", 'ship_country': 231
            }
        form = ContactInfoForm(data=data, shop=shop, contact=contact)
        self.assertEqual(True, form.is_valid())
        
        # bad state
        data = {
            'email': 'test_email@satchmoproject.com', 'first_name': 'Test', 'last_name': 'McTestalot','phone':'123-111-4411',

            'street1': "56 Cool Lane", 'city': "Niftyville", 'state': "ON", 'postal_code': "12344", 'country': 231,
            'ship_street1': "56 Industry Way", 'ship_city': "Niftytown", 'ship_state': "ON", 'ship_postal_code': "12366", 'ship_country': 231
            }
        form = ContactInfoForm(data=data, shop=shop, contact=contact)
        self.assertEqual(False, form.is_valid())
        
        # Canada
        CA = Country.objects.get(iso2_code__iexact="CA")
        
        # a valid one
        data = {
            'email': 'test_email@satchmoproject.com', 'first_name': 'Test', 'last_name': 'McTestalot','phone':'123-111-4411',

            'street1': "301 Front Street West", 'city': "Toronto", 'state': "ON", 'postal_code': "M5V 2T6", 'country': 39,
            'ship_street1': "301 Front Street West", 'ship_city': "Toronto", 'ship_state': "ON", 'ship_postal_code': "M5V 2T6", 'ship_country': 39
            }
        form = ContactInfoForm(data=data, shop=shop, contact=contact)
        self.assertEqual(True, form.is_valid())
        
        # bad province
        data = {
            'email': 'test_email@satchmoproject.com', 'first_name': 'Test', 'last_name': 'McTestalot','phone':'123-111-4411',

            'street1': "301 Front Street West", 'city': "Toronto", 'state': "NY", 'postal_code': "M5V 2T6", 'country': 39,
            'ship_street1': "301 Front Street West", 'ship_city': "Toronto", 'ship_state': "NY", 'ship_postal_code': "M5V 2T6", 'ship_country': 39
            }
        form = ContactInfoForm(data=data, shop=shop, contact=contact)
        self.assertEqual(False, form.is_valid())
        
        # bad postal code
        data = {
            'email': 'test_email@satchmoproject.com', 'first_name': 'Test', 'last_name': 'McTestalot','phone':'123-111-4411',

            'street1': "301 Front Street West", 'city': "Toronto", 'state': "ON", 'postal_code': "M5V 2TA", 'country': 39,
            'ship_street1': "301 Front Street West", 'ship_city': "Toronto", 'ship_state': "ON", 'ship_postal_code': "M5V 2TA", 'ship_country': 39
            }
        form = ContactInfoForm(data=data, shop=shop, contact=contact)
        self.assertEqual(False, form.is_valid())
        
        # Australia
        AU = Country.objects.get(iso2_code__iexact="AU")
        
        # a valid one
        data = {
            'email': 'test_email@satchmoproject.com', 'first_name': 'Test', 'last_name': 'McTestalot','phone':'123-111-4411',

            'street1': "Macquarie Street", 'city': "Sydney", 'state': "NSW", 'postal_code': "2000", 'country': 14,
            'ship_street1': "Macquarie Street", 'ship_city': "Sydney", 'ship_state': "NSW", 'ship_postal_code': "2000", 'ship_country': 14
            }
        form = ContactInfoForm(data=data, shop=shop, contact=contact)
        self.assertEqual(True, form.is_valid())
        
        # bad state
        data = {
            'email': 'test_email@satchmoproject.com', 'first_name': 'Test', 'last_name': 'McTestalot','phone':'123-111-4411',

            'street1': "Macquarie Street", 'city': "Sydney", 'state': "NY", 'postal_code': "2000", 'country': 14,
            'ship_street1': "Macquarie Street", 'ship_city': "Sydney", 'ship_state': "NY", 'ship_postal_code': "2000", 'ship_country': 14
            }
        form = ContactInfoForm(data=data, shop=shop, contact=contact)
        self.assertEqual(False, form.is_valid())
        
        # bad postal code
        data = {
            'email': 'test_email@satchmoproject.com', 'first_name': 'Test', 'last_name': 'McTestalot','phone':'123-111-4411',

            'street1': "Macquarie Street", 'city': "Sydney", 'state': "NSW", 'postal_code': "200A", 'country': 14,
            'ship_street1': "Macquarie Street", 'ship_city': "Sydney", 'ship_state': "NSW", 'ship_postal_code': "200A", 'ship_country': 14
            }
        form = ContactInfoForm(data=data, shop=shop, contact=contact)
        self.assertEqual(False, form.is_valid())




#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
ASyncIOController demo

"""
import asyncio
from notmm.utils.django_settings import LazySettings
from notmm.controllers.async import AsyncIOController
    
@asyncio.coroutine
def application(environ, start_response):

    settings = LazySettings()

    result = (yield from AsyncIOController(settings=settings)(environ, start_response))
    
    yield from result



    



from django.conf import settings
from django.contrib.sites.models import Site
from django.db import models
from django.db import transaction
from django.apps import apps
from django.utils.translation import ugettext_lazy as _
from keyedcache import cache_key, cache_get, cache_set, NotCachedError
from keyedcache.models import CachedObjectMixin
from livesettings.overrides import get_overrides
import logging

log = logging.getLogger('configuration.models')

__all__ = ['SettingNotSet', 'Setting', 'LongSetting', 'find_setting']

def _safe_get_siteid(site):
    if not site:
        try:
            site = Site.objects.get_current()
        except:
            transaction.rollback()
        if site and site.id:
            siteid = site.id
        else:
            siteid = settings.SITE_ID
    else:
        siteid = site.id
    transaction.commit()
    return siteid

#_safe_get_siteid=transaction.commit_manually(_safe_get_siteid)


def find_setting(group, key, site=None):
    """Get a setting or longsetting by group and key, cache and return it."""
       
    siteid = _safe_get_siteid(site)
    setting = None
    
    use_db, overrides = get_overrides(siteid)
    ck = cache_key('Setting', siteid, group, key)
    
    if use_db:
        try:
            setting = cache_get(ck)

        except NotCachedError, nce:
            if apps.apps_ready:
                try:
                    setting = Setting.objects.get(site__id__exact=siteid, key__exact=key, group__exact=group)

                except Setting.DoesNotExist:
                    # maybe it is a "long setting"
                    try:
                        setting = LongSetting.objects.get(site__id__exact=siteid, key__exact=key, group__exact=group)
           
                    except LongSetting.DoesNotExist:
                        pass
            
                cache_set(ck, value=setting)

    else:
        grp = overrides.get(group, None)
        if grp and grp.has_key(key):
            val = grp[key]
            setting = ImmutableSetting(key=key, group=group, value=val)
            log.debug('Returning overridden: %s', setting)
                
    if not setting:
        raise SettingNotSet(key, cachekey=ck)

    return setting

class SettingNotSet(Exception):    
    def __init__(self, k, cachekey=None):
        self.key = k
        self.cachekey = cachekey
        Exception.__init__(self, k, cachekey)

class SettingManager(models.Manager):
    def get_query_set(self):
        all = super(SettingManager, self).get_query_set()
        siteid = _safe_get_siteid(None)
        return all.filter(site__id__exact=siteid)


class ImmutableSetting(object):
    
    def __init__(self, group="", key="", value="", site=1):
        self.site = site
        self.group = group
        self.key = key
        self.value = value
        
    def cache_key(self, *args, **kwargs):
        return cache_key('OverrideSetting', self.site, self.group, self.key)
        
    def delete(self):
        pass
        
    def save(self, *args, **kwargs):
        pass
        
    def __repr__(self):
        return "ImmutableSetting: %s.%s=%s" % (self.group, self.key, self.value)


class Setting(models.Model, CachedObjectMixin):
    site = models.ForeignKey(Site, verbose_name=_('Site'))
    group = models.CharField(max_length=100, blank=False, null=False)
    key = models.CharField(max_length=100, blank=False, null=False)
    value = models.CharField(max_length=255, blank=True)

    objects = SettingManager()

    def __nonzero__(self):
        return self.id is not None

    def cache_key(self, *args, **kwargs):
        return cache_key('Setting', self.site, self.group, self.key)

    def delete(self):
        self.cache_delete()
        super(Setting, self).delete()

    def save(self, **kwargs):
        try:
            site = self.site
        except Site.DoesNotExist:
            self.site = Site.objects.get_current()
            
        super(Setting, self).save(**kwargs)
        
        self.cache_set()
        
    class Meta:
        unique_together = ('site', 'group', 'key')
        app_label = 'livesettings'

class LongSettingManager(models.Manager):
    def get_query_set(self):
        all = super(LongSettingManager, self).get_query_set()
        siteid = _safe_get_siteid(None)
        return all.filter(site__id__exact=siteid)

class LongSetting(models.Model, CachedObjectMixin):
    """A Setting which can handle more than 255 characters"""
    site = models.ForeignKey(Site, verbose_name=_('Site'))
    group = models.CharField(max_length=100, blank=False, null=False)
    key = models.CharField(max_length=100, blank=False, null=False)
    value = models.TextField(blank=True)

    objects = LongSettingManager()

    def __nonzero__(self):
        return self.id is not None

    def cache_key(self, *args, **kwargs):
        # note same cache pattern as Setting.  This is so we can look up in one check.
        # they can't overlap anyway, so this is moderately safe.  At the worst, the 
        # Setting will override a LongSetting.
        return cache_key('Setting', self.site, self.group, self.key)

    def delete(self):
        self.cache_delete()
        super(LongSetting, self).delete()

    def save(self, **kwargs):
        try:
            site = self.site
        except Site.DoesNotExist:
            self.site = Site.objects.get_current()
        super(LongSetting, self).save(**kwargs)
        self.cache_set()
        
    class Meta:
        unique_together = ('site', 'group', 'key')
        app_label = 'livesettings'



from django.http import HttpResponse
from livesettings import config_get_group
from payment.config import gateway_live

def auth_required(request):
    """
    Sends an authentication required response
    """
    response = HttpResponse(('Authorization Required'), mimetype="text/plain")
    response['WWW-Authenticate'] = 'Basic'
    response.status_code = 401
    return response

def get_cred():
    payment_module = config_get_group('PAYMENT_GOOGLE')
    live = gateway_live(payment_module)
    # get key and value
    if live:
        merchant_id = payment_module.MERCHANT_ID.value
        merchant_key = payment_module.MERCHANT_KEY.value
    else:
        merchant_id = payment_module.MERCHANT_TEST_ID.value
        merchant_key = payment_module.MERCHANT_TEST_KEY.value
    
    return (merchant_id, merchant_key)

def get_url():
    """
    Returns the urls needed
    """
    (merchant_id, merchant_key) = get_cred()
    payment_module = config_get_group('PAYMENT_GOOGLE')
    live = gateway_live(payment_module)
    if live:
        url_template = payment_module.POST_URL.value
    else:
        url_template = payment_module.POST_TEST_URL.value
    post_url = url_template % {'MERCHANT_ID' : merchant_id}
    return post_url

def do_auth(request):
    try:
        (merchant_id, merchant_key) = get_cred()
        # now get the http auth values
        http_auth = request.META['HTTP_AUTHORIZATION']
        (authmeth, auth) = http_auth.split(' ', 1)
        (username, password) = auth.strip().decode('base64').split(':', 1)
        if username != merchant_id:
            raise Exception()
        if password != merchant_key:
            raise  Exception()
        # passed auth
    except:
        return auth_required(request)


#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (C) 2007-2019 Etienne Robillard <tkadm30@yandex.ru>
# See NOTICE for details. All rights reserved.  
import logging, django
from importlib import import_module

from notmm.controllers.base import (
    BaseController, get_current_request, sessionmanager
    )
from notmm.utils.django_settings import SettingsProxy
from notmm.utils.django_compat import get_resolver

_logger = logging.getLogger(__name__)

__all__ = ('WSGIController',)

try:
    django.setup()
except:
    _logger.debug("django.setup() is disabled")


class WSGIController(BaseController):

    def __init__(self,
        settings=None,          #Django like settings module (default=$DJANGO_SETTINGS_MODULE)
        #New: dict based configuration (app_conf)
        #TODO document the changes in docs/config.txt
        app_conf={
            #'wsgi.request_class'        : HTTPRequest,     #WSGI Request middleware
            #'wsgi.response_class'       : HTTPResponse,    #WSGI Response middleware
            #'django.urlconf_class'      : None,           #Django URLConf module (default=$ROOT_URLCONF)
            'django.settings_autoload'  : True,          #Set this to False to disable Django settings autoloading (True)
            'logging_conf':None,      #Logging module to handle logging events at run-time  (experimental)
            'logging_instance':_logger,
            'logging.disabled': False,  #Set this to True to disable logging (experimental)
        }):

        """
        Initializes a ``BaseController`` instance for processing
        standard Django view functions and handling basic error conditions.

        Available keyword arguments:

        - ``settings``: Django settings module (optional)
        - ``app_conf``: Dict of configuration options. (optional)
        """
        # initialize Django module by importing Django settings
        # if True. (Bool)
        autoload = app_conf.get('django.settings_autoload', True)
        if settings is None:
            self.settings = SettingsProxy(autoload=autoload).get_settings()
        else:
            self.settings = settings

        # Setup basic logging to /var/log/django.log (default)
        logging_disabled = app_conf.get('logging.disabled', False)
        if not logging_disabled:
            self.logger = _logger
        else:
            #print('Logging is disabled!')
            self.logger = None

        # If using legacy autoload mecanism, attempt to register user-specified
        # wsgi callbacks.
        if (autoload and hasattr(self.settings, 'CUSTOM_ERROR_HANDLERS')):
            self.registerWSGIHandlers(self.settings.CUSTOM_ERROR_HANDLERS)

        if (autoload and 'ROOT_URLCONF' in self.settings):
            setattr(self, 'urlconf', self.settings.ROOT_URLCONF)
            setattr(self, 'resolver', get_resolver(self.urlconf))
            if not hasattr(self.resolver, '_urlconf_module'):
                self.resolver._urlconf_module = import_module(self.urlconf)
        else:
            self.resolver = None
            self.urlconf = None
        
        # sanity checks 
        if self.debug:
            assert self.urlconf != None, 'urlconf instance cannot be None!'
            assert self.resolver != None, 'resolver instance cannot be None!'

        # Do something with app_conf here
        self.app_conf = app_conf




'''
Canada Post Shipping Module
v0.1.1
'''

# Note, make sure you use decimal math everywhere!
from decimal import Decimal
from django.core.cache import cache
from django.template import loader, Context
from django.utils.safestring import mark_safe 
from django.utils.translation import ugettext as _
from livesettings import config_get_group, config_value
from shipping.modules.base import BaseShipper
import datetime
import logging
import urllib2

try:
    from xml.etree.ElementTree import fromstring, tostring
except ImportError:
    from elementtree.ElementTree import fromstring, tostring
    

log = logging.getLogger('canadapost.shipper')

class Shipper(BaseShipper):
    
    def __init__(self, cart=None, contact=None, service_type=None):

        self._calculated = False
        self.cart = cart
        self.contact = contact
        if service_type:    
            self.service_type_code = service_type[0]
            self.service_type_text = service_type[1]
        else:
            self.service_type_code = '99'
            self.service_type_text = 'Uninitialized'

        self.id = u'canadapost-%s' % (self.service_type_code)
  
    def __str__(self):
        '''
          This is mainly helpful for debugging purposes
        '''

        return 'Canada Post'

    def __unicode__(self):
        '''
          As is this.
        '''

        return 'Canada Post'
    
    def description(self):
        '''
          A basic description that will be displayed to the user when 
          selecting their shipping options
        '''

        return _('Canada Post - %s' % self.service_type_text)

    def cost(self):
        '''
          Complex calculations can be done here as long as the return 
          value is a decimal figure
        '''

        assert(self._calculated)
        return(Decimal(self.charges))

    def method(self):
        '''
          Describes the actual delivery service (Mail, FedEx, DHL, UPS, etc)
        '''

        return _('Canada Post')

    def expectedDelivery(self):
        '''
          Can be a plain string or complex calcuation 
          returning an actual date
        '''

        if str(self.delivery_days) <> '1':
            return _('%s business days' % self.delivery_days)
        else:
            return _('%s business day' % self.delivery_days)
    
    def valid(self, order=None):
        '''
        Can do complex validation about whether or not this
        option is valid. For example, may check to see if the 
        recipient is in an allowed country or location.
        '''

        return self.is_valid

    def _process_request(self, connection, request):
        '''
          Post the data and return the XML response
        '''
        conn = urllib2.Request(url=connection, data=request.encode("utf-8"))
        f = urllib2.urlopen(conn)
        all_results = f.read()
        self.raw = all_results
        return(fromstring(all_results))
    
    def calculate(self, cart, contact):
        '''
          Based on the chosen Canada Post method, we will do our call(s) 
          to Canada Post and see how much it will cost. We will also need 
          to store the results for further parsing and return via the
          methods above.
        '''
        log.debug("Starting Canada Post calculations")

        from satchmo.shop.models import Config
        settings =  config_get_group('shipping.modules.canadapost')

        verbose = settings.VERBOSE_LOG.value
            
        self.delivery_days = _('3 - 4') #Default setting for ground delivery
        shop_details = Config.objects.get_current()
        self.packaging = ''
        self.is_valid = False
        error = False
        self.charges = 0

        if not settings.CPCID.value:
            log.warn("No CPCID found in settings")
            return
        if settings.LIVE.value:
            connection = settings.CONNECTION.value
        else:
            connection = settings.CONNECTION_TEST.value
        
        configuration = {
            'cpcid': settings.CPCID.value,
            'turn_around_time': settings.TURN_AROUND_TIME.value,
            'packaging': self.packaging,
            'ship_type': self.service_type_code,
            'shop_details':shop_details,
        }
        c = Context({
                'config': configuration,
                'cart': cart,
                'contact': contact
            })
        
        t = loader.get_template('shipping/canadapost/request.xml')
        request = t.render(c)
        self.is_valid = False
        
        cache_key_response = "canadapost-cart-%s-response" % int(cart.id)
        cache_key_request = "canadapost-cart-%s-request" % int(cart.id)
        last_request = cache.get(cache_key_request)
        tree = cache.get(cache_key_response)
        
        if (last_request != request) or tree is None:
            self.verbose_log("Requesting from Canada Post [%s]\n%s", cache_key_request, request)
            cache.set(cache_key_request, request, 60)
            tree = self._process_request(connection, request)
            self.verbose_log("Got from Canada Post [%s]:\n%s", cache_key_response, self.raw)
            needs_cache = True
        else:
            needs_cache = False
            
        try:
            status_code = tree.getiterator('statusCode')
            status_val = status_code[0].text
            self.verbose_log("Canada Post Status Code for cart #%s = %s", int(cart.id), status_val)
        except AttributeError:
            status_val = "-1"
            

        if status_val == '1':
            self.is_valid = False
            self._calculated = False
            all_rates = tree.getiterator('product')

            for rate in all_rates:
                self.verbose_log("Got product id from cp: %s", rate.attrib['id'])
                if self.service_type_code == rate.attrib['id']:
                    self.charges = Decimal(rate.find('.//rate').text)
                    #YYYY-MM-DD
                    delivery_date = rate.find('.//deliveryDate').text
                    shipping_date = rate.find('.//shippingDate').text
                    self.delivery_days = datetime.date(
                                            int(delivery_date[:4]),
                                            int(delivery_date[5:7]),
                                            int(delivery_date[8:])) - \
                                         datetime.date(
                                            int(shipping_date[:4]),
                                            int(shipping_date[5:7]),
                                            int(shipping_date[8:]))
                    self.delivery_days = self.delivery_days.days
                    self.is_valid = True
                    self._calculated = True

            if not self.is_valid:
                self.verbose_log("Canada Post Cannot find rate for code: %s [%s]", self.service_type_code, self.service_type_text)
              
    def verbose_log(self, *args, **kwargs):
        if config_value('shipping.modules.canadapost', 'VERBOSE_LOG'):
            log.debug(*args, **kwargs)



# Copyright (C) 2005 by SICEm S.L. and Async Open Source
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

import gobject
import gtk

from gazpacho.loader.custom import combobox_set_content_from_string
from gazpacho.properties import prop_registry, CustomProperty, StringType
from gazpacho.widgetadaptor import WidgetAdaptor

class ComboBoxAdaptor(WidgetAdaptor):
    def create(self, context, interactive=True):
        combo = super(ComboBoxAdaptor, self).create(context, interactive)
        model = gtk.ListStore(gobject.TYPE_STRING)
        combo.set_model(model)
        return combo

    def fill_empty(self, context, widget):
        pass

# GtkComboBox

# XXX: This is too complex, fix properties API
#      TransparentProperty doesn't work, why?
class Items(CustomProperty, StringType):
    label = 'Items'
    default = ''
    def load(self):
        model = self.object.get_model()
        if not model:
            # Hack warning, only needed for GtkComboBoxEntry
            model = gtk.ListStore(str)
            self.object.set_model(model)
        self._value = '\n'.join([row[0] for row in model])
        self._initial = self.default

    def get(self):
        return self._value

    def set(self, value):
        self._value = value
        combobox_set_content_from_string(self.object, value)
prop_registry.override_property('GtkComboBox::items', Items)

# These properties are buggy in GTK+ 2.4
prop_registry.override_simple('GtkComboBox::column-span-column', editable=False)
prop_registry.override_simple('GtkComboBox::row-span-column', editable=False)

# GtkComboBoxEntry
prop_registry.override_simple('GtkComboBoxEntry::text-column', default=0)




from django.utils.translation import ugettext_lazy as _
from payment.modules.base import BasePaymentProcessor, ProcessorResult, NOTSET

class PaymentProcessor(BasePaymentProcessor):
    """
    Autosuccess Payment Module
    """
    def __init__(self, settings):
        super(PaymentProcessor, self).__init__('autosuccess', settings)

    def capture_payment(self, testing=False, order=None, amount=NOTSET):
        if not order:
            order = self.order

        if amount == NOTSET:
            amount = order.balance

        payment = self.record_payment(order=order, amount=amount, 
            transaction_id="AUTO", reason_code='0')

        return ProcessorResult(self.key, True, _('Success'), payment)



from decimal import Decimal
from l10n.utils import moneyfmt
import datetime

def get_product_quantity_adjustments(product, qty=1, parent=None):
    """Gets a list of adjustments for the price found for a product/qty"""

    qty_discounts = product.price_set.exclude(
        expires__isnull=False,
        expires__lt=datetime.date.today()).filter(quantity__lte=qty)

    # Get the price with the quantity closest to the one specified without going over
    adjustments = qty_discounts.order_by('price','-quantity', 'expires')[:1]
    if adjustments:
        adjustments = adjustments[0].adjustments(product)
    else:
        adjustments = None  
        if parent:
            adjustments = get_product_quantity_adjustments(parent, qty=qty)


    if not adjustments:
        adjustments = PriceAdjustmentCalc(None)

    return adjustments

def get_product_quantity_price(product, qty=Decimal('1'), delta=Decimal("0.00"), parent=None):
    """
    Returns price as a Decimal else None.
    First checks the product, if none, then checks the parent.
    """

    adjustments = get_product_quantity_adjustments(product, qty=qty, parent=parent)

    return adjustments.final_price()+delta

# -------------------------------------------
# helper objects - not Django model objects

class PriceAdjustmentCalc(object):
    """Helper class to handle adding up product pricing adjustments"""

    def __init__(self, price, product=None):
        self.price = price
        self.base_product = product
        self.adjustments = []

    def __add__(self, adjustment):
        self.adjustments.append(adjustment)
        return self

    def total_adjustment(self):
        total = Decimal(0)
        for adj in self.adjustments:
            total += adj.amount
        return total

    def _product(self):
        """Lazy product dereference"""
        if self.base_product:
            product = self.base_product
        else:
            product = self.price.product
        return product

    product = property(fget=_product)

    def final_price(self):
        total = Decimal(0)
        if self.price:
            total = self.price.price
            if total is None:
                total = Decimal(0)
        return total - self.total_adjustment()

class PriceAdjustment(object):
    """A single product pricing adjustment"""

    def __init__(self, key, label=None, amount=None):
        if label is None:
            label = key.capitalize()
        if amount is None:
            amount = Decimal(0)
        self.key = key
        self.label = label
        self.amount = amount

    def __unicode__(self):
        return u"%s: %s=%s" % (_('Price Adjustment'), self.label, moneyfmt(self.amount))




from django.contrib.sites.models import Site
from django.core.management.base import BaseCommand
from optparse import make_option
from product.models import Product, ProductPriceLookup

class Command(BaseCommand):
    help = "Builds Satcho Product pricing lookup tables."
    args = ['sitename...']

    requires_model_validation = True

    def handle(self, *sitenames, **options):
        from django.conf import settings

        verbosity = int(options.get('verbosity', 1))
        if len(sitenames) == 0:
            if verbosity>0:
                print "Rebuilding pricing for all products for all sites"
                sites = Site.objects.all()
        else:
            sites = []
            for sitename in sitenames:
                try:
                    sites.append(Site.objects.get(domain__iexact=sitename))
                except Site.DoesNotExist:
                    print "Warning: Could not find site '%s'" % sitename 

        total = 0
        for site in sites:
            ct = 0
            if verbosity > 0:
                print "Starting product pricing for %s" % site.domain

            if verbosity > 1:
                print "Deleting old pricing"
                
            for lookup in ProductPriceLookup.objects.filter(siteid=site.id):
                lookup.delete()
            
            products = Product.objects.active_by_site(site=site, variations=False)
            if verbosity > 0:
                print "Adding %i products" % products.count()
                
            for product in products:
                if verbosity > 1:
                    print "Processing product: %s" % product.slug
                
                prices = ProductPriceLookup.objects.smart_create_for_product(product)
                if verbosity > 1:
                    print "Created %i prices" % len(prices)
                
                ct += len(prices)
            
            if verbosity > 0:
                print "Added %i total prices for site" % ct
            
            total += ct

        if verbosity > 0:
            print "Added %i total prices" % total
            



# Copyright (C) 2004,2005 by SICEm S.L. and Imendio AB
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

import gtk

from gazpacho.placeholder import Placeholder
from gazpacho.properties import prop_registry
from gazpacho.gadget import Gadget
from gazpacho.widgetregistry import widget_registry
from gazpacho.widgets.base.window import WindowAdaptor

class DialogAdaptor(WindowAdaptor):
    def child_property_applies(self, context, ancestor, widget,
                               property_id):
        if property_id == 'response-id' and \
           isinstance(widget.parent, gtk.HButtonBox) and \
           isinstance(widget.parent.parent, gtk.VBox) and \
           widget.parent.parent.parent == ancestor:
            return True
        elif widget.parent == ancestor:
            return True

        return False

    def post_create(self, context, dialog, interactive=True):
        # set a reasonable default size for a dialog
        dialog.set_default_size(320, 260)

        # create the GladeWidgets for internal children
        self._setup_internal_children(dialog, context.get_project())

    def fill_empty(self, context, dialog):
        dialog.action_area.pack_start(Placeholder())
        dialog.action_area.pack_start(Placeholder())

        dialog.vbox.pack_start(Placeholder())

    def _setup_internal_children(self, dialog, project):
        child_class = widget_registry.get_by_name('GtkVBox')
        vbox_gadget = Gadget(child_class, project)
        vbox_gadget.setup_internal_widget(dialog.vbox, 'vbox',
                                          dialog.name or '')
        child_class = widget_registry.get_by_name('GtkHButtonBox')
        action_area_gadget = Gadget(child_class, project)
        action_area_gadget.setup_internal_widget(
            dialog.action_area,
            'action_area',
            dialog.name or '')
        self._init_property_meta_data(dialog)

    def _init_property_meta_data(self, dialog):
        """
        Set the metadata for some of the properties. This should not
        set default values of the properties since that is not always
        what we want.
        """
        # XXX not sure if we can assume that there actually is a vbox
        # and an action_area here?

        vbox = Gadget.from_widget(dialog.vbox)
        if vbox:
            vbox.get_prop('border-width').editable = False

        action_area = Gadget.from_widget(dialog.action_area)
        if action_area:
            action_area.get_prop('border-width').editable = False
            action_area.get_prop('spacing').editable = False

    def load(self, context, widget):
        gadget = super(WindowAdaptor, self).load(context, widget)
        self._init_property_meta_data(widget)
        return gadget

class MessageDialogAdaptor(DialogAdaptor):
    def post_create(self, context, dialog, interactive=True):
        dialog.set_default_size(400, 115)

# GtkDialog
prop_registry.override_simple('GtkDialog::has-separator', default=False)




from kiwi.ui.dialogs import info

from gazpacho.plugins import Plugin

class HelloPlugin(Plugin):

    def activate(self, app):
        self.signals = {}
        self.window = app.get_window()

        info('Hello Plugin Info: Hello Plugin activated!',
             parent=self.window)

        app.connect('set-project', self.on_app__set_project)
        app.connect('close-project', self.on_app__close_project)

    def deactivate(self):
        info('Hello Plugin Info: Hello Plugin deactivated',
             parent=self.window)

    def on_app__set_project(self, app, project):
        info('Hello Plugin Info: A new project is set: %s' % project.name,
             parent=self.window)

        # connect to anything we are interested on this project
        project.connect('changed', self.on_project__changed)

    def on_app__close_project(self, app, project):
        info('Hello Plugin Info: The project %s has been closed' %
             project.name,
             parent=self.window)

    def on_project__changed(self, project):
        info('Hello Plugin Info: The project %s has changed' % project.name,
             parent=self.window)



# Copyright (C) 2004,2005 by SICEm S.L. and Imendio AB
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

import gtk

from kiwi.component import get_utility

from gazpacho.app.bars import bar_manager
from gazpacho.i18n import _
from gazpacho.interfaces import IGazpachoApp

class Command(object):
    """A command is the minimum unit of undo/redoable actions.

    It has a description so the user can see it on the Undo/Redo
    menu items.

    Every Command subclass should implement the 'execute' method in
    the following way:
        - After a command is executed, if the execute method is called
        again, their effects will be undone.
        - If we keep calling the execute method, the action will
        be undone, then redone, then undone, etc...
        - To acomplish this every command constructor will probably
        need to gather more data that it may seems necessary.

    After you execute a command in the usual way you should put that
    command in the command stack of that project and that's what
    the push_undo method does. Otherwise no undo will be available.

    Some commands unifies themselves. This means that if you execute
    several commands of the same type one after the other, they will be
    treated as only one big command in terms of undo/redo. In other words,
    they will collapse. For example, every time you change a letter on a
    widget's name it is a command but all these commands unifies to one
    command so if you undo that all the name will be restored to the old one.
    """
    def __init__(self, description=None, ):
        self.description = description

    def __repr__(self):
        return '<Command %s>' % self.description

    def execute(self):
        """ This is the main method of the Command class.
        Note that it does not have any arguments so all the
        necessary data should be provided in the constructor.
        """
        pass

    def undo(self):
        """Convenience method that just call execute"""
        self.execute()

    def redo(self):
        """Convenience method that just call redo"""
        self.execute()

    def unifies(self, other):
        """True if self unifies with 'other'
        Unifying means that both commands can be treated as they
        would be only one command
        """
        return False

    def collapse(self, other):
        """Combine self and 'other' to form only one command.
        'other' should unifies with self but this method does not
        check that.
        """
        return False

class AccessorCommand(Command):
    """
    A simple Command which allows you to define a value to be set,
    a getter and a setter.
    """
    def __init__(self, description, value, setter, getter):
        """
        @param description: command description
        @param value: intial value to be set to the setter
        @param setter: a function taking one argument
        @param getter: a function returning a value and taking zero arguments.
        """
        Command.__init__(self, description)
        self._value = value
        self._setter = setter
        self._getter = getter

    def execute(self):
        oldvalue = self._getter()
        self._setter(self._value)
        self._value = oldvalue

class ContainerCommand(Command):
    """
    A command which can execute several other commands
    """
    def __init__(self, description, *commands):
        Command.__init__(self, description)
        self._commands = commands
        self._reverse = False

    def execute(self):
        commands = self._commands
        if self._reverse:
            commands = commands[::-1]

        for command in commands:
            command.execute()

        self._reverse = not self._reverse

class FlipFlopCommandMixin(object):
    """
    Base class for commands that alternatively call one method or another
    in their execute method based on a flipflop variable (two states)
    """

    def __init__(self, initial_value=True):
        self._state = initial_value

    def execute(self):
        if self._state:
            ret = self._execute_state1()
        else:
            ret = self._execute_state2()

        self._state = not self._state
        return ret

    def _execute_state1(self):
        """Subclasses should implement this method"""

    def _execute_state2(self):
        """Subclasses should implement this method"""


class CommandCreateDelete(FlipFlopCommandMixin, Command):
    def __init__(self, project, gadget, placeholder=None, parent=None,
                 create=True):
        FlipFlopCommandMixin.__init__(self, create)

        if create:
            description = _("Create %s") % gadget.name
        else:
            description = _("Delete %s") % gadget.name

        Command.__init__(self, description)

        self._project = project
        self._gadget = gadget
        self._placeholder = placeholder
        self._parent = parent
        self._initial_creation = create

    def _create_execute(self):
        from gazpacho.gadget import Gadget
        from gazpacho.placeholder import Placeholder

        gadget = self._gadget

        # Note that updating the dependencies might replace the
        # widget's gtk-widget so we need to make sure we refer to the
        # correct one afterward.
        gadget.deleted = False
        widget = gadget.widget

        if isinstance(widget, gtk.Window):

            # make window management easier by making created windows
            # transient for the editor window
            widget.set_transient_for(get_utility(IGazpachoApp).get_window())

            # Show windows earlier so we can restore the window-position
            # before the property editor is shown
            old_pos = widget.get_property('window-position')
            widget.set_position(gtk.WIN_POS_CENTER_ON_PARENT)
        else:
            if self._placeholder is None:
                for child in self._parent.widget.get_children():
                    if isinstance(child, Placeholder):
                        self._placeholder = child
                        break
            Gadget.replace(self._placeholder, widget, self._parent)

        self._gadget.project.add_widget(widget)
        self._gadget.select()

        widget.show_all()
        if isinstance(widget, gtk.Window):
            widget.set_position(old_pos)

            # we have to attach the accelerators groups so key shortcuts
            # keep working when this window has the focus. Only do
            # this the first time when creating a window, not when
            # redoing the creation since the accel group is already
            # set by then
            if self._initial_creation:
                widget.add_accel_group(bar_manager.get_accel_group())
                self._initial_creation = False

    _execute_state1 = _create_execute

    def _delete_execute(self):
        from gazpacho.gadget import Gadget
        from gazpacho.placeholder import Placeholder

        gadget = self._gadget

        if self._parent:
            if self._placeholder is None:
                self._placeholder = Placeholder()

            Gadget.replace(gadget.widget,
                           self._placeholder, self._parent)

        gadget.widget.hide()
        gadget.project.remove_widget(gadget.widget)
        gadget.deleted = True

    _execute_state2 = _delete_execute




# -*- coding: utf-8 -*-
"""Provides mixin objects to ease programming for translations."""
__docformat__="restructuredtext"

from django.contrib.sites.models import Site
from django.utils.translation import get_language
import keyedcache
import logging
from django.conf import settings

log = logging.getLogger('l10n.mixins')

class TranslatedObjectMixin(object):
    """Allows any object with a "translations" object to find the proper translation.
    """
    
    def _find_translation(self, language_code=None, attr='translations'):
        """Look up a translation for an attr.
        
        Ex: self._find_translation(language_code='en-us', attr='translations')
        """
        if not language_code:
            language_code = get_language()
            
        try:
            site = Site.objects.get_current()
            trans = keyedcache.cache_get([self.__class__.__name__, self.id], site=site, trans=attr, lang=language_code)
        except keyedcache.NotCachedError, nce:
            
            translations = getattr(self, attr)

            c = translations.filter(languagecode__exact = language_code)
            ct = c.count()

            if not c or ct == 0:
                pos = language_code.find('-')
                if pos>-1:
                    short_code = language_code[:pos]
                    #log.debug("%s: Trying to find root language content for: [%s]", self, short_code)
                    c = translations.filter(languagecode__exact = short_code)
                    ct = c.count()
                    if ct>0:
                        #log.debug("%s: Found root language content for: [%s]", self, short_code)
                        pass

            if not c or ct == 0:
                #log.debug("Trying to find default language content for: %s", self)
                c = translations.filter(languagecode__istartswith = settings.LANGUAGE_CODE)
                ct = c.count()

            if not c or ct == 0:
                #log.debug("Trying to find *any* language content for: %s", self)
                c = translations.all()
                ct = c.count()

            if ct > 0:
                trans = c[0]
            else:
                trans = None

            keyedcache.cache_set(nce.key, value=trans)

        return trans

# class ExampleTranslation(models.Model):
# 
#     menu = models.ForeignKey(Example, related_name="translations")
#     languagecode = models.CharField(_('language'), max_length=10, choices=settings.LANGUAGES)
#     title = models.CharField(_('title'), max_length=100, blank=False)
#     description = models.CharField(_('Description'), max_length=100, blank=True)
# 
#     class Meta:
#         ordering=('languagecode', )



from django.contrib.auth.decorators import user_passes_test
from django.core import urlresolvers
from django.http import HttpResponseRedirect
from django.shortcuts import render_to_response
from django.template import RequestContext
from django.utils.translation import ugettext as _
from product.forms import VariationManagerForm, InventoryForm, ProductExportForm, ProductImportForm
from product.models import Product
from satchmo_utils.views import bad_or_missing
import logging

log = logging.getLogger('product.views.adminviews')

def edit_inventory(request):
    """A quick inventory price, qty update form"""
    if request.method == "POST":
        new_data = request.POST.copy()
        form = InventoryForm(new_data)
        if form.is_valid():
            form.save(request)
            url = urlresolvers.reverse('edit_inventory')
            return HttpResponseRedirect(url)
    else:
        form = InventoryForm()

    ctx = RequestContext(request, {
        'title' : _('Inventory Editor'),
        'form' : form
        })

    return render_to_response('product/admin/inventory_form.html',
                              context_instance=ctx)

edit_inventory = user_passes_test(lambda u: u.is_authenticated() and u.is_staff, login_url='/accounts/login/')(edit_inventory)

def export_products(request, template='product/admin/product_export_form.html'):
    """A product export tool"""
    if request.method == 'POST':
        new_data = request.POST.copy()
        form = ProductExportForm(new_data)
        if form.is_valid():
            return form.export(request)
    else:
        form = ProductExportForm()
        fileform = ProductImportForm()  
        

    ctx = RequestContext(request, {
        'title' : _('Product Import/Export'),
        'form' : form,
        'importform': fileform
        })

    return render_to_response(template, context_instance=ctx)

export_products = user_passes_test(lambda u: u.is_authenticated() and u.is_staff, login_url='/accounts/login/')(export_products)

def import_products(request, maxsize=10000000):  
    """ 
    Imports product from an uploaded file.
    """  

    if request.method == 'POST':  
        errors = []
        results = []
        if 'upload' in request.FILES:  
            infile = request.FILES['upload']              
            form = ProductImportForm()
            results, errors = form.import_from(infile, maxsize=maxsize)
                       
        else:
            errors.append('File: %s' % request.FILES.keys())
            errors.append(_('No upload file found'))
                 
        ctx = RequestContext(request, {
            'errors' : errors,
            'results' : results
        })
        return render_to_response("product/admin/product_import_result.html",
                                  context_instance=ctx)
    else:
        url = urlresolvers.reverse('product_export')
        return HttpResponseRedirect(url)

import_products = user_passes_test(lambda u: u.is_authenticated() and u.is_staff, login_url='/accounts/login/')(import_products)
        
# def product_active_report(request):
#     
#     products = Product.objects.filter(active=True)
#     products = [p for p in products.all() if 'productvariation' not in p.get_subtypes]
#     ctx = RequestContext(Request, {title: 'Active Product Report', 'products' : products })
#     return render_to_response('product/admin/active_product_report.html', ctx)
#     
# product_active_report = user_passes_test(lambda u: u.is_authenticated() and u.is_staff, login_url='/accounts/login/')(product_active_report)    

def variation_list(request):
    products = [p for p in Product.objects.all() if "ConfigurableProduct" in p.get_subtypes()]
    ctx = RequestContext(request, {
           'products' : products,
    })

    return render_to_response('product/admin/variation_manager_list.html',
                              context_instance=ctx)


def variation_manager(request, product_id = ""):
    try:
        product = Product.objects.get(id=product_id)
        subtypes = product.get_subtypes()
        
        if 'ProductVariation' in subtypes:
            # got a variation, we want to work with its parent
            product = product.productvariation.parent.product
            if 'ConfigurableProduct' in product.get_subtypes():
                url = urlresolvers.reverse("variation_manager", 
                    kwargs = {'product_id' : product.id})
                return HttpResponseRedirect(url)
            
        if 'ConfigurableProduct' not in subtypes:
            return bad_or_missing(request, _('The product you have requested is not a Configurable Product.'))
            
    except Product.DoesNotExist:
            return bad_or_missing(request, _('The product you have requested does not exist.'))
     
    if request.method == 'POST':
        new_data = request.POST.copy()       
        form = VariationManagerForm(new_data, product=product)
        if form.is_valid():
            log.debug("Saving form")
            form.save(request)
            # rebuild the form
            form = VariationManagerForm(product=product)
        else:
            log.debug('errors on form')
    else:
        form = VariationManagerForm(product=product)
    
    ctx = RequestContext(request, {
        'product' : product,
        'form' : form,
    })
    return render_to_response('product/admin/variation_manager.html',
                              context_instance=ctx)

variation_manager = user_passes_test(lambda u: u.is_authenticated() and u.is_staff, login_url='/accounts/login/')(variation_manager)



import logging

from django.utils.translation import ugettext_lazy as _
from livesettings import *

log = logging.getLogger('tieredweight.config')

from shipping.config import SHIPPING_ACTIVE

SHIPPING_ACTIVE.add_choice(('shipping.modules.tieredweight', _('Tiered Weight Shipping')))

log.debug('loaded')


from decimal import Decimal

class Processor(object):
    
    method="no"
    
    def __init__(self, order=None, user=None):
        """
        Any preprocessing steps should go here
        For instance, copying the shipping and billing areas
        """
        pass

    def by_product(self, product, quantity=Decimal('1')):
        return Decimal("0.0")
        
    def by_orderitem(self, orderitem):
        return Decimal("0.0")
                
    def by_price(self, taxclass, price):
        return Decimal("0.0")                
                
    def shipping(self, product, user):
        return Decimal("0.0")                
                
    def process(self, order=None):
        """
        Calculate the tax and return it
        """
        return Decimal("0.0"), {}

    def get_percent(self, *args, **kwargs):
        return Decimal("0")

    def get_rate(self, *args, **kwargs):
        return Decimal("0")
        


"""
UPS Shipping Module
You must have a UPS account to use this module.
You may register at ups.com

This module uses the XML online tools for maximum flexibility.  It is
primarily created for use in the US but reconfiguring for international
shipping or more advanced uses should be straightforward.

It is recommended that you refer to the UPS shipper developer documents
(available when you register at UPS) in order to tailor this to your
unique needs.
"""

from decimal import Decimal
from django.core.cache import cache
from django.template import Context, loader
from django.utils.translation import ugettext as _
from livesettings import config_get_group, config_value
from shipping import signals
from shipping.modules.base import BaseShipper
import logging
import urllib2

try:
    from xml.etree.ElementTree import fromstring, tostring
except ImportError:
    from elementtree.ElementTree import fromstring, tostring

log = logging.getLogger('ups.shipper')
class Shipper(BaseShipper):
    
    def __init__(self, cart=None, contact=None, service_type=None):
        self._calculated = False
        self.cart = cart
        self.contact = contact
        if service_type:        
            self.service_type_code = service_type[0]
            self.service_type_text = service_type[1]
        else:
            self.service_type_code = "99"
            self.service_type_text = "Uninitialized"
        self.id = u"UPS-%s-%s" % (self.service_type_code, self.service_type_text)
        self.raw = "NO DATA"
        #if cart or contact:
        #    self.calculate(cart, contact)
    
    def __str__(self):
        """
        This is mainly helpful for debugging purposes
        """
        return "UPS"
        
    def description(self):
        """
        A basic description that will be displayed to the user when selecting their shipping options
        """
        return _("UPS - %s" % self.service_type_text)

    def cost(self):
        """
        Complex calculations can be done here as long as the return value is a decimal figure
        """
        assert(self._calculated)
        return(Decimal(self.charges))

    def method(self):
        """
        Describes the actual delivery service (Mail, FedEx, DHL, UPS, etc)
        """
        return _("UPS")

    def expectedDelivery(self):
        """
        Can be a plain string or complex calcuation returning an actual date
        """
        if self.delivery_days <> "1":
            return _("%s business days" % self.delivery_days)
        else:
            return _("%s business day" % self.delivery_days)
        
    def valid(self, order=None):
        """
        Can do complex validation about whether or not this option is valid.
        For example, may check to see if the recipient is in an allowed country
        or location.
        """
        return self.is_valid

    def _process_request(self, connection, request):
        """
        Post the data and return the XML response
        """
        conn = urllib2.Request(url=connection, data=request.encode("utf-8"))
        f = urllib2.urlopen(conn)
        all_results = f.read()
        self.raw = all_results
        return(fromstring(all_results))
        
    def calculate(self, cart, contact):
        """
        Based on the chosen UPS method, we will do our call to UPS and see how much it will
        cost.
        We will also need to store the results for further parsing and return via the
        methods above
        """
        from satchmo_store.shop.models import Config
        
        settings =  config_get_group('shipping.modules.ups')
        self.delivery_days = _("3 - 4") #Default setting for ground delivery
        shop_details = Config.objects.get_current()
        configuration = {
            'xml_key': settings.XML_KEY.value,
            'account': settings.ACCOUNT.value,
            'userid': settings.USER_ID.value,
            'password': settings.USER_PASSWORD.value,
            'container': settings.SHIPPING_CONTAINER.value,
            'pickup': settings.PICKUP_TYPE.value,
            'ship_type': self.service_type_code,
            'shop_details':shop_details,
        }
        shippingdata = {
                'config': configuration,
                'cart': cart,
                'contact': contact,
                'shipping_address' : shop_details,
                'shipping_phone' : shop_details.phone,
                'shipping_country_code' : shop_details.country.iso2_code
        }
        signals.shipping_data_query.send(Shipper, shipper=self, cart=cart, shippingdata=shippingdata)
        c = Context(shippingdata)
        t = loader.get_template('shipping/ups/request.xml')
        request = t.render(c)
        self.is_valid = False
        if settings.LIVE.value:
            connection = settings.CONNECTION.value
        else:
            connection = settings.CONNECTION_TEST.value
        cache_key_response = "ups-cart-%s-response" % int(cart.id)
        cache_key_request = "ups-cart-%s-request" % int(cart.id)
        last_request = cache.get(cache_key_request)
        tree = cache.get(cache_key_response)

        if (last_request != request) or tree is None:
            self.verbose_log("Requesting from UPS [%s]\n%s", cache_key_request, request)
            cache.set(cache_key_request, request, 60)
            tree = self._process_request(connection, request)
            self.verbose_log("Got from UPS [%s]:\n%s", cache_key_response, self.raw)
            needs_cache = True
        else:
            needs_cache = False

        try:
            status_code = tree.getiterator('ResponseStatusCode')
            status_val = status_code[0].text
            self.verbose_log("UPS Status Code for cart #%s = %s", int(cart.id), status_val)
        except AttributeError:
            status_val = "-1"
        
        if status_val == '1':
            self.is_valid = False
            self._calculated = False
            all_rates = tree.getiterator('RatedShipment')
            for response in all_rates:
                if self.service_type_code == response.find('.//Service/Code/').text:
                    self.charges = response.find('.//TotalCharges/MonetaryValue').text
                    if response.find('.//GuaranteedDaysToDelivery').text:
                        self.delivery_days = response.find('.//GuaranteedDaysToDelivery').text
                    self.is_valid = True
                    self._calculated = True
                    if needs_cache:
                        cache.set(cache_key_response, tree, 60)
                        
            if not self.is_valid:
                self.verbose_log("UPS Cannot find rate for code: %s [%s]", self.service_type_code, self.service_type_text)
        
        else:
            self.is_valid = False
            self._calculated = False

            try:
                errors = tree.find('.//Error')
                log.info("UPS %s Error: Code %s - %s" % (errors[0].text, errors[1].text, errors[2].text))
            except AttributeError:
                log.info("UPS error - cannot parse response:\n %s", self.raw)
            
    def verbose_log(self, *args, **kwargs):
        if config_value('shipping.modules.ups', 'VERBOSE_LOG'):
            log.debug(*args, **kwargs)
        
        



#!/usr/bin/env python
# Copyright (C) Etienne Robillard 2007-2019 <tkadm30@yandex.com>
# All rights reserved.
"""
Elixir Controller API (Elixir WSGI App)

Helper class to implement a generic WSGI handler supporting the
SQLElixir ORM on top of SQLAlchemy.

This module uses the ``ScopedSession`` class to provide persistent
connections to SQL databases using the ``threadlocal`` method.
"""

from importlib import import_module

# sqlalchemy required for Elixir to work! :-)
from sqlalchemy import create_engine, MetaData
from sqlalchemy.orm import scoped_session, sessionmaker

from wsgi import WSGIController

elixir_module = import_module('elixir')

__all__ = ['ElixirController']

class ElixirController(WSGIController):
    """
    A ``BaseController`` subclass supporting contextual sessions
    on top of Elixir/SQLAlchemy.
    """

    def __init__(self, engine_name, db_options={
        'echo': True,
        'strategy': 'threadlocal'
        }, **kwargs):
        """Setup the default Elixir parameters here"""

        self.db_options = db_options

        super(ElixirController, self).__init__(**kwargs)

        # if self.debug: assert 'settings' in self

        # Configure the Engine instance unless already defined
        self.set_engine(engine_name)
        # Prepare the database session to handle requests
        self.init_session(self.engine)
        # Set up the metadata obj
        self.metadata = MetaData(self.engine)

    def get_engine(self):
        """Returns a fully configured SQLAlchemy engine instance"""
        return getattr(self, '_engine')

    def set_engine(self, engine_name):
        """Setup a initial engine instance"""
        engine = create_engine(engine_name, **self.db_options)
        setattr(self, '_engine', engine)

        return None

    engine = property(get_engine, set_engine)

    def init_session(self, engine, shortnames=True, autoflush=True,
        autocommit=False):
        """
        Prepare the session backend and initialize ORM mapping
        for Elixir."""
        Session = scoped_session(sessionmaker(autoflush=autoflush))
        #Session.set_session(engine, autoflush=autoflush, autocommit=autocommit)

        ## replace the elixir session with our own
        ## http://cleverdevil.org/computing/68/
        elixir_module.session = Session
        elixir_module.options_defaults.update(dict(shortnames=bool(shortnames)))

        # setup_all
        if not elixir_module.metadata.is_bound():
            elixir_module.metadata.bind = engine

        # create tables if they're missing...
        # meta.create_all()

        # copy elixir.session to self.session
        # setattr(self, '_session', elixir.session)

    def __call__(self, environ, start_response):
        return self.application(environ, start_response)



from django import forms
from django.core.exceptions import ImproperlyConfigured
from payment import signals
from payment.forms import SimplePayShipForm
from payment.modules.purchaseorder.models import PurchaseOrder
from satchmo_utils import app_enabled
from signals_ahoy import signals

class PurchaseorderPayShipForm(SimplePayShipForm):
    po_number = forms.CharField(max_length=20, required=False)
    
    def __init__(self, *args, **kwargs):
        if not app_enabled('purchaseorder'):
            raise ImproperlyConfigured('To use Purchase Order payment methods, you must have payment.modules.purchaseorder in your INSTALLED_APPS')

        super(PurchaseorderPayShipForm, self).__init__(*args, **kwargs)
    
    def save(self, request, cart, contact, payment_module):
        """Save the order and the po information for this orderpayment"""
        signals.form_presave.send(PurchaseorderPayShipForm, form=self)
        super(PurchaseorderPayShipForm, self).save(request, cart, contact, payment_module)
        data = self.cleaned_data
        po = PurchaseOrder(po_number=data.get('po_number', ''), order=self.order)
        po.save()
        self.purchaseorder = po
        signals.form_postsave.send(PurchaseorderPayShipForm, form=self)




from django.core.management.base import NoArgsCommand
import os
import string

def module_to_dict(module, omittable=lambda k: k.startswith('_')):
    "Converts a module namespace to a Python dictionary. Used by get_settings_diff."
    return dict([(k, repr(v)) for k, v in module.__dict__.items() if not omittable(k)])

class Command(NoArgsCommand):
    help = "Delete all databases! Be careful about using this!"
    
    def handle_noargs(self, **options):
        """Delete the old database."""
        from django.conf import settings
        # Because settings are imported lazily, we need to explicitly load them.
        settings._import_settings()
        engine = settings.DATABASE_ENGINE
        db_host = settings.DATABASE_HOST 
        db_port = settings.DATABASE_PORT 
        db_pass = settings.DATABASE_PASSWORD 
        db_name = settings.DATABASE_NAME 
        db_user = settings.DATABASE_USER 
        
        response_erase_all = string.lower(raw_input("Type 'yes' to erase ALL of the data in your %s db named %s: " % (engine,db_name)))
        if response_erase_all != 'yes':
            print "Aborting..."
            return None
        if engine == 'sqlite3':
            try:
                os.unlink(db_name)
            except OSError:
                pass
        elif engine == 'mysql':
            import _mysql
            s = _mysql.connect(host=db_host,
                               user=db_user,
                               passwd=db_pass)
            for cmd in ['drop database if exists %s',
                        'create database %s CHARACTER SET utf8 COLLATE utf8_general_ci']:
                s.query(cmd % db_name)

        elif engine in ("postgresql_psycopg2", "postgresql"):
            if db_name == '':
                raise AssertionError("You must specify a value for DATABASE_NAME in local_settings.py.")
            if db_user == '':
                raise AssertionError("You must specify a value for DATABASE_USER in local_settings.py.")
            params=" --username=%s  --password" % db_user
            if db_host:
                params += " --host=%s" % db_host
            if db_port:
                params += " --port=%s" % db_port
            params += " %s" % db_name
            print("""You will be prompted for the password for the user '%s' twice.
            Once to drop the existing database and then a second time to create
            the database.""" % db_user)
            for cmd in ['dropdb %s', 'createdb %s']:
                os.system(cmd % params)
        else:
            raise AssertionError("Unknown database engine %s" % engine)



# Some helper functions to get by without Python 2.4

# set
try:
    set = set
except NameError:
    from sets import Set as set

orig_cmp = cmp
# [].sort
def sort_list(l, cmp=None, key=None, reverse=False):
    try:
        l.sort(cmp, key, reverse)
    except TypeError, e:
        if not str(e).startswith('sort expected at most 1 arguments'):
            raise
        if cmp is None:
            cmp = orig_cmp
        if key is not None:
            # the cmp=cmp parameter is required to get the original comparator
            # into the lambda namespace
            cmp = lambda self, other, cmp=cmp: cmp(key(self), key(other))
        if reverse:
            cmp = lambda self, other, cmp=cmp: -cmp(self,other)
        l.sort(cmp)

# sorted
try:
    sorted = sorted
except NameError:
    # global name 'sorted' doesn't exist in Python2.3
    # this provides a poor-man's emulation of the sorted built-in method
    def sorted(l, cmp=None, key=None, reverse=False):
        sorted_list = list(l)
        sort_list(sorted_list, cmp, key, reverse)
        return sorted_list

# rsplit
try:
    ''.rsplit
    def rsplit(s, delim, maxsplit):
        return s.rsplit(delim, maxsplit)

except AttributeError:
    def rsplit(s, delim, maxsplit):
        """Return a list of the words of the string s, scanning s
        from the end. To all intents and purposes, the resulting
        list of words is the same as returned by split(), except
        when the optional third argument maxsplit is explicitly
        specified and nonzero. When maxsplit is nonzero, at most
        maxsplit number of splits - the rightmost ones - occur,
        and the remainder of the string is returned as the first
        element of the list (thus, the list will have at most
        maxsplit+1 elements). New in version 2.4.
        >>> rsplit('foo.bar.baz', '.', 0)
        ['foo.bar.baz']
        >>> rsplit('foo.bar.baz', '.', 1)
        ['foo.bar', 'baz']
        >>> rsplit('foo.bar.baz', '.', 2)
        ['foo', 'bar', 'baz']
        >>> rsplit('foo.bar.baz', '.', 99)
        ['foo', 'bar', 'baz']
        """
        assert maxsplit >= 0

        if maxsplit == 0: return [s]

        # the following lines perform the function, but inefficiently.
        #  This may be adequate for compatibility purposes
        items = s.split(delim)
        if maxsplit < len(items):
            items[:-maxsplit] = [delim.join(items[:-maxsplit])]
        return items



from shipping.modules.tieredquantity.models import Carrier, QuantityTier, CarrierTranslation
from django.contrib import admin
from django.utils.translation import get_language, ugettext_lazy as _

class QuantityTier_Inline(admin.TabularInline):
    model = QuantityTier
    extra = 5
    ordering = ('quantity', 'expires')

class CarrierTranslation_Inline(admin.TabularInline):
    model = CarrierTranslation
    extra = 1

class CarrierOptions(admin.ModelAdmin):
    ordering = ('key',)
    inlines = [CarrierTranslation_Inline, QuantityTier_Inline]

admin.site.register(Carrier, CarrierOptions)




from satchmo_store.shop.models import Config, Cart, CartItem, CartItemDetails, Order, OrderItem, OrderItemDetail, OrderStatus, OrderPayment, OrderPaymentFailure, OrderAuthorization, OrderVariable, OrderTaxDetail
from django.contrib import admin
from django.utils.translation import ugettext_lazy as _
from satchmo_utils.admin import AutocompleteAdmin
from satchmo_utils.widgets import ReadOnlyWidget

class CartItem_Inline(admin.TabularInline):
    model = CartItem
    extra = 3
    raw_id_fields = ('product',)

class CartItemDetails_Inline(admin.StackedInline):
    model = CartItemDetails
    extra = 1

class ConfigOptions(admin.ModelAdmin):
    list_display = ('site', 'store_name')
    filter_horizontal = ('shipping_countries',)
    fieldsets = (
        (None, {'fields': (
            'site', 'store_name', 'store_description')
            }),
        (_('Store Contact'), {'fields' : (
            'store_email', 'phone', 'street1', 'street2',
            'city', 'state', 'postal_code', 'country',)
            }),
        (_('Shipping Countries'), {'fields' : (
            'in_country_only', 'sales_country', 'shipping_countries')
            })
    )

class CartOptions(admin.ModelAdmin):
    list_display = ('date_time_created','numItems','total')
    inlines = [CartItem_Inline]

class CartItemOptions(admin.ModelAdmin):
    inlines = [CartItemDetails_Inline]

class OrderItem_Inline(admin.TabularInline):
    model = OrderItem
    extra = 3
    raw_id_fields = ('product',)

class OrderItemDetail_Inline(admin.TabularInline):
    model = OrderItemDetail
    extra = 3

class OrderAuthorizationDetail_Inline(admin.TabularInline):
    model = OrderAuthorization
    extra = 0

class OrderPaymentDetail_Inline(admin.TabularInline):
    model = OrderPayment
    extra = 0

class OrderPaymentFailureDetail_Inline(admin.TabularInline):
    model = OrderPaymentFailure
    extra = 0

class OrderStatus_Inline(admin.StackedInline):
    model = OrderStatus
    extra = 1

class OrderVariable_Inline(admin.TabularInline):
    model = OrderVariable
    extra = 1

class OrderTaxDetail_Inline(admin.TabularInline):
    model = OrderTaxDetail
    extra = 1

class OrderOptions(AutocompleteAdmin):
    fieldsets = (
        (None, {'fields': ('site', 'contact', 'method', 'status', 'discount_code', 'notes')}), (_('Shipping Method'), {'fields':
            ('shipping_method', 'shipping_description')}), (_('Shipping Address'), {'classes': ('collapse',), 'fields':
            ('ship_street1', 'ship_street2', 'ship_city', 'ship_state', 'ship_postal_code', 'ship_country')}), (_('Billing Address'), {'classes': ('collapse',), 'fields':
            ('bill_street1', 'bill_street2', 'bill_city', 'bill_state', 'bill_postal_code', 'bill_country')}), (_('Totals'), {'fields':
            ('sub_total', 'shipping_cost', 'shipping_discount', 'tax', 'discount', 'total', 'time_stamp')}))
    list_display = ('id', 'contact', 'time_stamp', 'order_total', 'balance_forward', 'status', 'invoice', 'packingslip', 'shippinglabel')
    list_filter = ['time_stamp', 'contact', 'status']
    date_hierarchy = 'time_stamp'
    list_filter = ['time_stamp', 'status']
    date_hierarchy = 'time_stamp'
    search_fields = ('contact__first_name', 'contact__last_name', 'contact__email')
    related_search_fields = {'contact': ('first_name', 'last_name', 'email')}
    related_string_functions = {'contact': lambda c: u"%s &lt;%s&gt;" % (c.full_name, c.email)}
    inlines = [OrderItem_Inline, OrderStatus_Inline, OrderVariable_Inline,
        OrderTaxDetail_Inline, OrderAuthorizationDetail_Inline,
        OrderPaymentDetail_Inline, OrderPaymentFailureDetail_Inline]

    def formfield_for_dbfield(self, db_field, **kwargs):
        read_only = ['status']
        field = super(OrderOptions, self).formfield_for_dbfield(db_field, **kwargs)
        if db_field.name in read_only:
            field.widget = ReadOnlyWidget()
        return field


class OrderItemOptions(admin.ModelAdmin):
    inlines = [OrderItemDetail_Inline]

class OrderPaymentOptions(admin.ModelAdmin):
    list_filter = ['payment']
    list_display = ['id', 'order', 'payment', 'amount_total', 'time_stamp']
    fieldsets = (
        (None, {'fields': ('order', 'payment', 'amount', 'transaction_id', 'time_stamp')}), )
    raw_id_fields = ['order']


class OrderAuthorizationOptions(OrderPaymentOptions):
    list_display = ['id', 'order', 'capture', 'payment', 'amount_total', 'complete', 'time_stamp']
    fieldsets = (
        (None, {'fields': ('order', 'capture', 'payment', 'amount', 'transaction_id', 'complete', 'time_stamp')}), )

admin.site.register(Cart, CartOptions)
admin.site.register(CartItem, CartItemOptions)
admin.site.register(Config, ConfigOptions)
admin.site.register(Order, OrderOptions)
admin.site.register(OrderItem, OrderItemOptions)
admin.site.register(OrderPayment, OrderPaymentOptions)
admin.site.register(OrderAuthorization, OrderAuthorizationOptions)



from decimal import Decimal
from django.contrib.sites.models import Site
from django.core import urlresolvers
from django.test import TestCase
from django.test.client import Client
from product.models import *
from satchmo_store.contact.models import *
from satchmo_store.shop.models import *
import keyedcache

class TestRecurringBilling(TestCase):
    fixtures = ['l10n-data.yaml', 'test_shop.yaml', 'sub_products.yaml', 'config', 'initial_data.yaml']

    def setUp(self):
        self.customer = Contact.objects.create(first_name='Jane', last_name='Doe')
        US = Country.objects.get(iso2_code__iexact="US")
        self.customer.addressbook_set.create(street1='123 Main St', city='New York', state='NY', postal_code='12345', country=US)
        import datetime
        site = Site.objects.get_current()
        for product in Product.objects.all():
            price, expire_length = self.getTerms(product)
            if price is None:
                continue
            order = Order.objects.create(contact=self.customer, shipping_cost=0, site=site)
            order.orderitem_set.create(
                product=product,
                quantity=Decimal('1'),
                unit_price=price,
                line_item_price=price,
                expire_date=datetime.datetime.now() + datetime.timedelta(days=expire_length),
                completed=True
            )
            order.recalculate_total()
            order.payments.create(order=order, payment='DUMMY', amount=order.total)
            order.save()

    def tearDown(self):
        keyedcache.cache_delete()

    def testProductType(self):
        product1 = Product.objects.get(slug='membership-p1')
        product2 = Product.objects.get(slug='membership-p2')
        self.assertEqual(product1.subscriptionproduct.expire_length, 21)
        self.assertEqual(product1.subscriptionproduct.recurring, True)
        self.assertEqual(product1.subscriptionproduct.recurring, True)
        self.assertEqual(product1.price_set.all()[0].price, Decimal('3.95'))
        self.assertEqual(product2.subscriptionproduct.get_trial_terms(0).expire_length, 7)

    def testCheckout(self):
       pass

    def testCronRebill(self):
        for order in OrderItem.objects.all():
            price, expire_length = self.getTerms(order.product)
            if price is None:
                continue
            self.assertEqual(order.expire_date, datetime.date.today() + datetime.timedelta(days=expire_length))
            #set expire date to today for upcoming cron test
            order.expire_date = datetime.date.today()
            order.save()

        order_count = OrderItem.objects.count()
        self.c = Client()
        url = urlresolvers.reverse('satchmo_cron_rebill')
        self.response = self.c.get(url)
        self.assertEqual(self.response.status_code, 200)
        self.assertEqual(self.response.content, 'Authentication Key Required')

        self.response = self.c.get(url, data={'key': config_value('PAYMENT','CRON_KEY')})
        self.assertEqual(self.response.status_code, 200)
        self.assertEqual(self.response.content, '')
        self.assert_(order_count < OrderItem.objects.count())
        self.assertEqual(order_count, OrderItem.objects.count()/2.0)
        for order in OrderItem.objects.filter(expire_date__gt=datetime.datetime.now()):
            price, expire_length = self.getTerms(order.product, ignore_trial=True)
            if price is None:
                continue
            self.assertEqual(order.expire_date, datetime.date.today() + datetime.timedelta(days=expire_length))
            self.assertEqual(order.order.balance, Decimal('0.00'))

    def getTerms(self, object, ignore_trial=False):
        if object.subscriptionproduct.get_trial_terms().count() and ignore_trial is False:
            price = object.subscriptionproduct.get_trial_terms(0).price
            expire_length = object.subscriptionproduct.get_trial_terms(0).expire_length
            return price, expire_length
        elif object.is_subscription or ignore_trial is True:
            price = object.price_set.all()[0].price
            expire_length = object.subscriptionproduct.expire_length
            return price, expire_length
        else:
            return None, None #it's not a recurring object so no test will be done




from shipping.modules.productshipping.models import Carrier, CarrierTranslation, ProductShippingPrice
from product.models import Product
from product.admin import ProductOptions
from django.contrib import admin
from django.utils.translation import get_language, ugettext_lazy as _

class CarrierTranslation_Inline(admin.TabularInline):
    model = CarrierTranslation
    ordering = ('carrier',)
    extra = 1

class CarrierOptions(admin.ModelAdmin):
    ordering = ('key',)
    inlines = [CarrierTranslation_Inline]

class ProductShippingPriceInline(admin.TabularInline):
    model = ProductShippingPrice

admin.site.register(Carrier, CarrierOptions)
admin.site.unregister(Product)
ProductOptions.inlines.append(ProductShippingPriceInline)
admin.site.register(Product, ProductOptions)




# Copyright (C) 2004,2005 by SICEm S.L.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

import gobject
import gtk

class CommandView(gtk.ScrolledWindow):
    """
    This class is just a little TreeView that knows how
    to show the command stack of a project.
    It shows a plain list of all the commands performed by
    the user and also it mark the current command that
    would be redone if the user wanted so.
    Older commands are under newer commands on the list.
    """
    def __init__(self):
        gtk.ScrolledWindow.__init__(self)
        self._project = None

        self._model = gtk.ListStore(bool, str)
        self._treeview = gtk.TreeView(self._model)
        self._treeview.set_headers_visible(False)

        column = gtk.TreeViewColumn()
        renderer1 = gtk.CellRendererPixbuf()
        column.pack_start(renderer1, expand=False)
        column.set_cell_data_func(renderer1, self._draw_redo_position)

        renderer2 = gtk.CellRendererText()
        column.pack_start(renderer2, expand=True)
        column.add_attribute(renderer2, 'text', 1)

        self._treeview.append_column(column)

        self.add(self._treeview)

    def set_project(self, project):
        self._project = project
        self.update()

    def update(self):
        self._model.clear()
        if self._project is None:
            return

        undos = self._project.undo_stack.get_undo_commands()
        if undos:
            for cmd in undos[:-1]:
                self._model.insert(0, (False, cmd.description))
            self._model.insert(0, (True, undos[-1].description))

        for cmd in self._project.undo_stack.get_redo_commands():
            self._model.insert(0, (False, cmd.description))

    def _draw_redo_position(self, column, cell, model, iter):
        is_the_one = model[iter][0]

        if is_the_one:
            stock_id = gtk.STOCK_JUMP_TO
        else:
            stock_id = None

        cell.set_property('stock-id', stock_id)

gobject.type_register(CommandView)



from django.conf.urls.defaults import *

#Urls which need to be loaded at root level.
adminpatterns = patterns('',
    (r'^admin/print/(?P<doc>[-\w]+)/(?P<id>\d+)', 
        'shipping.views.displayDoc', {}, 
        'satchmo_print_shipping'),
)



from django.contrib import admin
from product.modules.configurable.models import ConfigurableProduct, ProductVariation

class ProductVariationOptions(admin.ModelAdmin):
    filter_horizontal = ('options',)

admin.site.register(ConfigurableProduct)
admin.site.register(ProductVariation, ProductVariationOptions)





# Copyright (C) 2004,2005 by SICEm S.L.
#               2006 by Async Open Source
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

import gtk
from kiwi.environ import environ
from kiwi.python import Settable
from kiwi.ui.dialogs import BaseDialog
from kiwi.ui.objectlist import ObjectList, Column
from kiwi.utils import gsignal, type_register

from gazpacho.constants import MISSING_ICON, STOCK_SIZEGROUP
from gazpacho.i18n import _

(STOCK_ID,
 STOCK_LABEL) = list(range(2))

_stock_icons = []

def create_stock_icon(iconfile, stock_id, icon_factory=None):
    if not icon_factory:
        icon_factory = gtk.IconFactory()

    icon_source = gtk.IconSource()
    icon_source.set_filename(iconfile)
    icon_set = gtk.IconSet()
    icon_set.add_source(icon_source)
    icon_factory.add(stock_id, icon_set)

    return icon_factory

def register_stock_icons():
    """
    Register the custom stock icons used by gazpacho
    """
    icon_factory = None

    for fname, stock_id in [('gazpacho-icon.png', MISSING_ICON),
                            ('sizegroup-both.png', STOCK_SIZEGROUP)]:
        filename = environ.find_resource('pixmap', fname)
        icon_factory = create_stock_icon(filename, stock_id, icon_factory)

    icon_factory.add_default()

def get_stock_icons():
    """
    Get a sorted list of all available stock icons
    @returns: a list of tuples; (stock_label, stock_id)
    """

    global _stock_icons
    if _stock_icons:
        return _stock_icons

    icons = []

    for stock_id in gtk.stock_list_ids():
        # Do not list stock ids starting with gazpacho-, they
        # are considered private
        if stock_id.startswith('gazpacho-'):
            continue

        stock_info = gtk.stock_lookup(stock_id)
        if not stock_info:
            # gtk-new -> New
            if stock_id.startswith('gtk-'):
                stock_label = stock_id[4:].replace('-', ' ')
            else:
                stock_label = stock_id
            stock_label = stock_label.capitalize()
        else:
            stock_label = stock_info[1].replace('_', '')

        icons.append((stock_label, stock_id))

    icons.sort()
    _stock_icons = icons

    return icons

class StockIconList(gtk.HBox):
    """
    I am a helper to manage stock icons
    """

    gsignal('changed', str)

    def __init__(self):
        gtk.HBox.__init__(self)

        self._model = gtk.ListStore(str, str)
        self._populate_icons()

        self._create_ui()

    def _create_ui(self):
        self._preview = gtk.Image()
        self._preview.set_sensitive(False)
        self.pack_start(self._preview, False, False, 6)

        self._stock = gtk.ComboBoxEntry(self._model)
        self._stock.clear()
        self._stock.connect('changed', self._on_stock_id_changed)
        self._stock.set_sensitive(False)
        self._stock.set_wrap_width(5)

        renderer = gtk.CellRendererPixbuf()
        renderer.set_property('xalign', 0.0)
        self._stock.pack_start(renderer)
        self._stock.add_attribute(renderer, 'stock-id', STOCK_ID)

        renderer = gtk.CellRendererText()
        renderer.set_property('xalign', 0.0)
        self._stock.pack_start(renderer)
        self._stock.add_attribute(renderer, 'text', STOCK_LABEL)

        self.pack_start(self._stock, False, False)

    def _populate_icons(self):
        model = self._model

        for stock_label, stock_id in get_stock_icons():
            model.append((stock_id, stock_label))

    def _on_stock_id_changed(self, combobox):
        stock_id = self._stock.child.get_text()
        if not stock_id in get_stock_icons():
            stock_id = MISSING_ICON

        self._preview.set_from_stock(stock_id, gtk.ICON_SIZE_BUTTON)
        self.emit('changed', stock_id)

    def set_sensitive(self, sensitive):
        """
        Sets the stock icon list to sensitive
        @param sensitive: True if sensitive
        @type sensitive: boolean
        """
        self._preview.set_sensitive(sensitive)
        self._stock.set_sensitive(sensitive)

    def reset(self):
        """
        Resets the senstivty and selects the first item
        """
        self._stock.set_active(0)
        self.set_sensitive(False)

    def set_stock_id(self, stock_id):
        """
        Sets a stock icon id from a string
        @param stock_id: stock icon id
        @type stock_id: string
        """
        if not stock_id in get_stock_icons():
            preview = MISSING_ICON
        else:
            for row in self._model:
                if row[STOCK_ID] == stock_id:
                    self._stock.set_active_iter(row.iter)
                    break
            else:
                raise AssertionError
            preview = stock_id

        self._stock.child.set_text(stock_id)
        self._preview.set_from_stock(preview, gtk.ICON_SIZE_BUTTON)

    def get_stock_id(self):
        """
        @returns: the current stock id
        """
        return self._stock.child.get_text()

type_register(StockIconList)

class StockIconDialog(BaseDialog):
    def __init__(self, parent=None):
        BaseDialog.__init__(self, parent, title=_('Stock Icons'),
                            buttons=(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
                                     gtk.STOCK_OK, gtk.RESPONSE_OK))
        self.set_size_request(260, 330)

        self._stockicons = ObjectList([Column('stock_id', use_stock=True),
                                       Column('name')])
        self._stockicons.set_headers_visible(False)
        self._stockicons.connect('row-activated',
                                 self._on_stockicons__row_activated)
        self._icons = {}
        for stock_label, stock_id in get_stock_icons():
            icon = Settable(stock_id=stock_id, name=stock_label)
            self._stockicons.append(icon)
            self._icons[stock_id] = icon

        self.vbox.pack_start(self._stockicons)
        self._stockicons.show()

    def select(self, value):
        icon = self._icons.get(value)
        if icon:
            self._stockicons.select(icon)

    def get_selected(self):
        icon = self._stockicons.get_selected()
        if icon:
            return icon.stock_id

    def _on_stockicons__row_activated(self, objectlist, icon):
        self.emit('response', gtk.RESPONSE_OK)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os

from notmm.controllers.schevo  import SchevoController
from notmm.utils.configparse   import loadconf
from notmm.utils.wsgilib       import HTTPRequest, HTTPResponse
from werkzeug.test import create_environ
from test_support import (
    BaseControllerTestCase, TestClient,
    settings,
    setup_test_handlers
    )

def get_app_conf(section):
    return loadconf('development.ini', section=section)

class SchevoControllerTestCase(BaseControllerTestCase):

    wsgi_app = SchevoController
    response_class = HTTPResponse

    def __init__(self, methodName='runTest'):

        super(BaseControllerTestCase, self).__init__(methodName)

        self.request = HTTPRequest(create_environ())

    def setUp(self):
        self.callback = self.wsgi_app(
            #the HTTP request obj for testing
            self.request,
            settings.SCHEVO['DATABASE_NAME']
            )
        setup_test_handlers(self.callback, settings)
        self.client = TestClient(self.callback)

    def tearDown(self):
        self.callback = None

    def test_with_schevo_database(self):
        response = self.client.post('/test_post')
        self.assertEqual(response[1] == '200 OK', True)





"""
Allow Satchmo to use an email address instead of the user id as the
primary id
Taken from a posting on the Django mailing list.
Thanks to Vasily Sulatskov for sending this to the list.
"""

from django.contrib.auth.backends import ModelBackend
from django.contrib.auth.models import User
import re

email_re = re.compile(
    r"(^[-!#$%&'*+/=?^_`{}|~0-9A-Z]+(\.[-!#$%&'*+/=?^_`{}|~0-9A-Z]+)*"  # dot-atom
    r'|^"([\001-\010\013\014\016-\037!#-\[\]-\177]|\\[\001-\011\013\014\016-\177])*"' # quoted-string
    r')@(?:[A-Z0-9-]+\.)+[A-Z]{2,6}$', re.IGNORECASE)  # domain
            
class EmailBackend(ModelBackend):
    """Authenticate using email only"""
    def authenticate(self, username=None, password=None):
        #If username is an email address, then try to pull it up
        if email_re.search(username):
            user = User.objects.filter(email__iexact=username)
            if user.count() > 0:
                user = user[0]
                if user.check_password(password):
                    return user
        return None
            



# Copyright (C) 2006 by Nokia Corporation
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

import gtk

from gazpacho.commandmanager import command_manager

# This module contains very high level handy functions
# that are just wrappers around internal gazpacho machinery
#
# It should be easy to import this module from anyware so
# local imports in these functions are not only allowed but
# recommended to avoid dependency cycles
#
# This module is similar to the zapi module in Zope 3

def create_gadget(project, adaptor, placeholder=None, parent=None,
                  interactive=True):
    """
    Creates a gadget and add it to a project.

    Basically it does all of the following:

      1. Create the Gtk+ object

      2. Wrap it in a Gadget

      3. Add it to a project

      4. Make all the process undoable by using a command

    @param project: The project the gadget will be added to
    @type project: gazpacho.project.Project
    @param adaptor: The widget adaptor that will be used to create and wrapp
      the widget
    @type adaptor: subclass of gazpacho.widgetadaptor.WidgetAdaptor
    @param placeholder: the placeholder that will be replaced by the gadget.
      It can be None if the adaptor is for a toplevel widget
    @type placeholder: gazpacho.placeholder.Placeholder
    @param parent: the parent of the gadget it's going to be created. It can
      be None for widgets that don't have a parent (e.g. toplevels)
    @type parent: gazpacho.gadget.Gadget
    @param interactive: this boolean indicates if the user can be prompted
      for additional options while creating the gadget. For tests it should
      be False
    @type interactive: boolean
    @return: the gadget that was created
    @rtype: gazpatcho.widget.Gadget
    """
    from gazpacho import util
    from gazpacho.command import CommandCreateDelete
    from gazpacho.palette import palette
    from gazpacho.widgetadaptor import CreationCancelled

    if placeholder:
        parent = util.get_parent(placeholder)
        if parent is None:
            return

    if project is None:
        project = parent.project

    gadget = None
    try:
        gadget = _create_gadget(project, adaptor, interactive)
    except CreationCancelled:
        return

    cmd = CommandCreateDelete(project, gadget, placeholder, parent, True)
    command_manager.execute(cmd, project)

    if not palette.persistent_mode:
        palette.unselect_widget()

    if not isinstance(gadget.widget, gtk.Window):
        gadget.get_prop('visible').value = True

    return gadget

def delete_gadget(project, gadget):
    """
    Delete a gadget

    Delete a gadget and remove it from the project. Note that the gadget won't
    probably be destroyed for undo purposes

    @param project: The project the gadget belongs to
    @type project: gazpacho.project.Project
    @param gadget: the gadget to be removed
    @type gadget: gazpacho.gadget.Gadget
    """
    # internal children cannot be deleted. Should we notify the user?
    if gadget.internal_name is not None:
        return

    if isinstance(gadget.widget, gtk.TreeViewColumn):
        from gazpacho.util import get_parent
        from gazpacho.widgets.base.treeview import CommandAddRemoveTreeViewColumn
        tree_view = get_parent(gadget.widget)
        cmd = CommandAddRemoveTreeViewColumn(tree_view, gadget, project, False)
    else:
        from gazpacho.command import CommandCreateDelete
        cmd = CommandCreateDelete(project, gadget, None, gadget.get_parent(),
                                  False)

    command_manager.execute(cmd, project)

# Private internal functions

def _create_gadget(project, adaptor, interactive):
    from gazpacho.gadget import Gadget
    gadget = Gadget(adaptor, project)
    gadget.create_widget(interactive)
    return gadget



"""
WSGI config for django111_pypy59 project.

It exposes the WSGI callable as a module-level variable named ``application``.

For more information on this file, see
https://docs.djangoproject.com/en/1.11/howto/deployment/wsgi/
"""

import os
#import django

from django.core.wsgi import get_wsgi_application
#from django.core.handlers.wsgi import WSGIHandler

os.environ.setdefault("DJANGO_SETTINGS_MODULE", "benchmark.settings")
#django.setup()

#def application(environ, start_response):
#    return WSGIHandler()(environ, start_response)
application = get_wsgi_application()




# -*- coding: utf-8 -*-
__docformat__="restructuredtext"
from django.shortcuts import render_to_response
from django.template import RequestContext
from satchmo_utils import is_string_like
from satchmo_store.shop.models import Cart

def cart_has_minimum_order(template="product/minimum_order.html", min_order="PAYMENT.MINIMUM_ORDER"):
    """
    Decorator for checkout views that ensures the active cart meets the minimum order
    requirements.  If not, then it shows the user the minimum order required.
    
    Params:
    - template: defaults to satchmo/product/minimum_order.html
    - min_order: If this is a string, it wil be used to look up the value from Satchmo's configuration
      system, if it is a decimal, it will be used directly as the minimum required.
    """
    
    def _decorate(view_func):
        def _checkorder(request, template=template, min_order=min_order, *args, **kwargs):
            # resolve the min order
            if is_string_like(min_order):
                from livesettings import config_value
                min_order = config_value(*min_order.split('.'))
            
            cart = Cart.objects.from_request(request)
            if cart.total >= min_order:
                return view_func(request, *args, **kwargs)
            else:
                ctx = RequestContext(request, {'minimum_order' : min_order})
                return render_to_response(template, context_instance=ctx)

        _checkorder.__doc__ = view_func.__doc__
        _checkorder.__dict__ = view_func.__dict__

        return _checkorder
    return _decorate



import os.path

from notmm.utils.urlmap import RegexURLMap,url

from django.contrib import admin
admin.autodiscover()

urlpatterns = RegexURLMap()

# home.py views
urlpatterns.include('satchmo_store.shop.urls', prefix='^shop/')

# debug views
#urlpatterns.add_routes('',    
#    (r'^media/(?P<path>.*)$', 'django.views.static.serve',
#        {'document_root': os.path.join(os.path.dirname(__file__), 'media/')})
#    )

urlpatterns.include('product.urls.base')
urlpatterns.include('product.urls.category', prefix='^category/')
urlpatterns.include('product.urls.products', prefix='^product/')
urlpatterns.include('livesettings.urls', prefix='^admin/livestore_settings/')

# django admin views
urlpatterns.include(admin.site.urls, prefix='^admin/')

urlpatterns.commit()

__all__ = ['urlpatterns']




import re

from django.utils.translation import ugettext as _

def validate(postcode):
    """
    Validates Australian postal codes.
    """
    postcode = postcode.strip()
    if postcode.isdigit():
        return postcode
    else:
        raise ValueError(_('Invalid Australian postal code'))



# -*- coding: utf-8 -*-

from django.utils.translation import ugettext_lazy as _
from livesettings import *
from satchmo_utils import load_module
from satchmo_store.shop import get_satchmo_setting

PRODUCT_GROUP = ConfigurationGroup('PRODUCT', _('Product Settings'))

PRODUCT_TYPES = config_register(MultipleStringValue(PRODUCT_GROUP,
    'PRODUCT_TYPES',
    description=_("Product Model Options"),
    default=['product::ConfigurableProduct', 'product::ProductVariation'],
    choices=[('product::ConfigurableProduct', _('Configurable Product')),
             ('product::ProductVariation', _('Product Variation')),
             ('product::CustomProduct', _('Custom Order Product')),
             ('product::SubscriptionProduct', _('Subscription Product')),
             ('product::DownloadableProduct', _('Downloadable Product'))]
    ))

config_register(
    StringValue(PRODUCT_GROUP,
        'IMAGE_DIR',
        description=_("Upload Image Dir"),
        help_text=_("""Directory name for storing uploaded images.
    This value will be appended to MEDIA_ROOT.  Do not worry about slashes.
    We can handle it any which way."""),
        default="images")
)

config_register_list(
    StringValue(PRODUCT_GROUP,
        'PROTECTED_DIR',
        description=_("Protected dir"),
        help_text=_("""This is only used if you use Downloadable Products.
This value will be appended to MEDIA_ROOT.  Do not worry about slashes.
We can handle it any which way."""),
        default="protected",
        requires=PRODUCT_TYPES,
        requiresvalue='product::DownloadableProduct'
    ),

    PositiveIntegerValue(PRODUCT_GROUP,
        'NUM_DISPLAY',
        description=_("Total featured"),
        help_text=_("Total number of featured items to display"),
        default=20
    ),

    PositiveIntegerValue(PRODUCT_GROUP,
        'NUM_PAGINATED',
        description=_("Number featured"),
        help_text=_("Number of featured items to display on each page"),
        default=10
    ),

    MultipleStringValue(PRODUCT_GROUP,
        'MEASUREMENT_SYSTEM',
        description=_("Measurement System"),
        help_text=_("Default measurement system to use for products."),
        choices=[('metric',_('Metric')),
                    ('imperial',_('Imperial'))],
        default="imperial"
    ),
    
    BooleanValue(PRODUCT_GROUP,
        'NO_STOCK_CHECKOUT',
        description=_("Allow checkout with 0 inventory?"),
        help_text=_("If yes, then customers can buy even if your inventory is 0 for a product."),
        default=True
    ),
    
    BooleanValue(PRODUCT_GROUP,
        'RANDOM_FEATURED',
        description= _("Random Display"),
        help_text= _("Enable random display of featured products on home page"),
        default=False
    ),

    BooleanValue(PRODUCT_GROUP,
        'TRACK_INVENTORY',
        description=_("Track inventory levels?"),
        help_text=_("If no, then inventory will not be tracked for products sold."),
        default=True
    ),
)



# Copyright (C) 2004,2005 by SICEm S.L. and Imendio AB
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

from gazpacho.properties import prop_registry, CustomProperty, StringType
from gazpacho.widgets.base.base import ContainerAdaptor

class TextViewAdaptor(ContainerAdaptor):
    def fill_empty(self, context, gadget):
        pass

class TextProp(CustomProperty, StringType):
    def get(self):
        buffer = self.object.get_buffer()
        (start, end) = buffer.get_bounds()
        return buffer.get_text(start, end, False)

    def set(self, value):
        buffer = self.object.get_buffer()
        buffer.set_text(value or '')

prop_registry.override_property('GtkTextView::text', TextProp)



# Copyright (C) 2004,2005 by SICEm S.L.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

import gtk
import gobject

from kiwi.utils import gsignal

from gazpacho.app.bars import bar_manager
from gazpacho.commandmanager import command_manager
from gazpacho.popup import Popup
from gazpacho.gadget import Gadget
from gazpacho.util import select_iter, get_all_children
from gazpacho.signalhandlers import SignalHandlerStorage

(GADGET_COLUMN, N_COLUMNS) = list(range(2))
COLUMN_TYPES = [object]

class WidgetTreeView(gtk.ScrolledWindow):
    gsignal('selection-changed', object)
    def __init__(self, app):
        gtk.ScrolledWindow.__init__(self)

        self._app = app

        self.set_shadow_type(gtk.SHADOW_IN)

        self._project = None

        # We need to know the signal id which we are connected to so
        # that when the project changes we stop listening to the old
        # project
        self._handlers = SignalHandlerStorage()

        # The model
        self._model = gtk.TreeStore(*COLUMN_TYPES)

        # The view
        self._tree_view = gtk.TreeView(self._model)
        self._tree_view.connect('row-activated',
                                self._on_treeview_row_activated)
        self._tree_view.connect('key-press-event', self._on_treeview_key_press)
        #self._tree_view.connect('motion-notify-event', self._motion_notify_cb)
        self._tree_view.connect('button-press-event',
                                self._on_treeview_button_press)
        self._tree_view.set_headers_visible(False)
        self._tree_view.show()

        self._tree_selection = self._tree_view.get_selection()
        self._tree_selection.connect('changed', self._on_treeselection_changed)
        self._tree_selection.set_mode(gtk.SELECTION_MULTIPLE)
        self._add_columns()

        # True when we are going to set the project selection. So that
        # we don't recurse cause we are also listening for the project
        # changed selection signal
        self._updating_selection = False

        self.add(self._tree_view)

    #
    # Public methods
    #

    def set_project(self, project):
        """
        Set the current project.

        @param project: the current project
        @type project: L{gazpacho.project.Project}
        """
        if project == self._project:
            return
        # Thie view stops listening to the old project(if there was one)
        if self._project:
            self._handlers.disconnect_all()

        # Set to null while we remove all the items from the store,
        # because we are going to trigger selection_changed signal
        # emisions on the View. By setting the project to None,
        # _on_treeselection_changed will just return
        self._project = None

        self._model.clear()

        # if we were passed None we are done
        if not project:
            return

        self._project = project
        self._populate_model()

        # Here we connect to all the signals of the project that interest us
        self._handlers.connect(project, 'add-gadget',
                               self._on_project_add_gadget)
        self._handlers.connect(project, 'remove-gadget',
                               self._on_project_remove_gadget)
        self._handlers.connect(project, 'gadget-name-changed',
                               self._on_project_gadget_name_changed)
        self._handlers.connect(project.selection, 'selection-changed',
                               self._on_selection_changed)

    #
    # Private methods
    #

    def _populate_model(self):
        if not self._project:
            return

        # add the widgets and recurse
        self._populate_model_real(self._project.get_windows(), None, True)

    def _populate_model_real(self, widgets, parent_iter, add_children):
        for w in widgets:
            iter = parent_iter
            gadget = Gadget.from_widget(w)
            if gadget:
                iter = self._model.append(parent_iter, (gadget,))
                if add_children:
                    children = gadget.get_children()
                    self._populate_model_real(children, iter, True)

    def _add_columns(self):
        column = gtk.TreeViewColumn()
        renderer = gtk.CellRendererPixbuf()
        column.pack_start(renderer, False)
        column.set_cell_data_func(renderer, self._view_cell_function, True)

        renderer = gtk.CellRendererText()
        column.pack_start(renderer)
        column.set_cell_data_func(renderer, self._view_cell_function, False)

        self._tree_view.append_column(column)

    def _view_cell_function(self, tree_column, cell, model, iter, is_icon):
        widget = model[iter][GADGET_COLUMN]
        if not widget:
            return

        if is_icon:
            if widget.adaptor.pixbuf:
                cell.set_property('pixbuf', widget.adaptor.pixbuf)
            else:
                cell.set_property('pixbuf', None)
        else:
            cell.set_property('text', widget.name)

    def _find_iter(self, iter, findme):
        next = iter
        while next:
            gadget = self._model[next][GADGET_COLUMN]
            if not gadget:
                print ('Could not get the gadget from the model')

            # Check for widget wrapper and real widgets
            if gadget == findme or gadget.widget == findme:
                return next

            if self._model.iter_has_child(next):
                child = self._model.iter_children(next)
                retval = self._find_iter(child, findme)
                if retval:
                    return retval

            next = self._model.iter_next(next)

        return None

    def _find_iter_by_widget(self, widget):
        root = self._model.get_iter_root()
        return self._find_iter(root, widget)

    def _add_children(self, parent, parent_iter):
        """ This function is needed because sometimes when adding a widget
        to the tree their children are not added.

        Please note that not all widget has a gadget associated with them
        so there are some special cases we need to handle."""
        parent_gadget = Gadget.from_widget(parent)
        if parent_gadget:
            children = parent_gadget.get_children()
        else:
            children = get_all_children(parent)

        for child in children:
            gadget = Gadget.from_widget(child)
            if not gadget:
                if isinstance(child, gtk.Container):
                    self._add_children(child, parent_iter)
                else:
                    continue

            elif not self._find_iter(parent_iter, gadget):
                child_iter = self._model.append(parent_iter, (gadget,))
                self._add_children(child, child_iter)

    def _get_selected_gadgets(self):
        """
        Get a list of the gadgets selected in the widget tree.

        @return: list of selected gadgets
        @rtype list (of L{gazpacho.gadget.Gadget})
        """
        if not self._project:
            return []

        # There are no cells selected
        model, iters = self._tree_selection.get_selected_rows()
        gadgets = []
        for iter in iters:
            gadget = model[iter][GADGET_COLUMN]
	    # Workaround for bug #344275
            if gadget:
                gadgets.append(gadget)
        return gadgets

    #
    # Signal handlers
    #

    def _on_treeview_button_press(self, view, event):
        """
        Show the context menu on a right click.
        """
        if not self._project:
            return False

        if event.button != 3:
            return False

        result = view.get_path_at_pos(int(event.x), int(event.y))
        if not result:
            return False
        path = result[0]
        row = self._model[path]
        widget = row[GADGET_COLUMN]
        select_iter(self._tree_view, row.iter)

        popup = Popup(command_manager, widget)
        popup.pop(event)
        return True

    def _on_treeselection_changed(self, selection):
        """
        Update the widget selection when the selection in the treeview
        changes.
        """
        # the project might not be set when the treeview is modified
        if not self._project:
            return

        # we are already trying to update the selection, no need to do
        # it again.
        if self._updating_selection:
            return

        gadgets = self._get_selected_gadgets()
        self.emit('selection-changed', gadgets)

        self._updating_selection = True

        self._project.selection.clear(False)
        for gadget in gadgets:
            self._project.selection.add(gadget.widget, False)
        self._project.selection.selection_changed()
        self._updating_selection = False

    def _on_treeview_row_activated(self, treeview, path, column):
        """
        Show the toplevel widget to which the widget on the activated
        row belongs.
        """
        gadget = self._model[path][GADGET_COLUMN]
        toplevel = gadget.widget.get_toplevel()
        toplevel.show_all()
        toplevel.present()

    def _on_treeview_key_press(self, view, event):
        """
        Delete the selected gadget if the delete key has been pressed.
        """
        if event.keyval in (gtk.keysyms.Delete, gtk.keysyms.KP_Delete):
            bar_manager.activate_action('Delete')

    def _on_project_add_gadget(self, project, gadget):
        """
        When the widget is added to the project it should be added to
        the widget tree as well.
        """
        parent = gadget.get_parent()
        parent_iter = None
        if parent:
            parent_iter = self._find_iter_by_widget(parent)

        # When creating a widget with internal children (e.g. GtkDialog) the
        # children are added first so they do have a gadget parent but
        # don't have a parent_iter (since the parent has not being added yet
        if not parent_iter and parent:
            # we need to add this widget when adding the parent
            return

        gadget_iter = self._model.append(parent_iter, (gadget,))

        # now check for every children of this widget if it is already on the
        # tree. If not we add it.
        self._add_children(gadget.widget, gadget_iter)

        self._updating_selection = True
        gadget.select()
        self._updating_selection = False

        select_iter(self._tree_view, gadget_iter)

    def _on_project_remove_gadget(self, project, gadget):
        """
        Remove the widget from the widget tree when it's removed from
        the project.
        """
        iter = self._find_iter_by_widget(gadget)
        if iter:
            self._model.remove(iter)

    def _on_project_gadget_name_changed(self, project, widget):
        """
        Update the name of the widget in the widget tree when the name
        of the widget is changed.
        """
        treeiter = self._find_iter_by_widget(widget)
        # TODO: Figure out why treeiter is None here, it happens under
        #       unknown circumstances, see bug 333034
        if treeiter:
            model = self._model
            model.row_changed(model[treeiter].path, treeiter)

    def _on_selection_changed(self, selection):
        """
        Update the widget tree when the widget selection changes.
        """
        if self._updating_selection:
            return

        self._updating_selection = True

        self._tree_selection.unselect_all()
        for widget in selection:
            gadget_iter = self._find_iter_by_widget(widget)
            if gadget_iter:
                select_iter(self._tree_view, gadget_iter)

        self._updating_selection = False

    # This is not working anyway because the current
    # button_press_event code is not creating the widget when you
    # click on a parent having a palette button selected
    #
    #def _motion_notify_cb(self, view, event):
    #    if pw.project_window is None:
    #        cursor.set_for_widget_adaptor(event.window, None)
    #    else:
    #        cursor.set_for_widget_adaptor(event.window,
    #                                      pw.project_window.add_class)
    #
    #    return False

gobject.type_register(WidgetTreeView)



from django import forms
from satchmo_store.contact import signals
import logging

log = logging.getLogger('contact.listeners')

def au_postcode_validator(sender, postcode=None, country=None, **kwargs):
    if country.iso2_code == 'AU':
        from l10n.validators import aupostcode
        try:
            pc = aupostcode.validate(postcode)
            return pc
        except ValueError, ve:
            raise forms.ValidationError('Please enter a valid Australian postal code.')
signals.validate_postcode.connect(au_postcode_validator)


def ca_postcode_validator(sender, postcode=None, country=None, **kwargs):
    if country.iso2_code == 'CA':
        from l10n.validators import capostcode
        try:
            pc = capostcode.validate(postcode)
            return pc
        except ValueError, ve:
            raise forms.ValidationError('Please enter a valid Canadian postal code.')
signals.validate_postcode.connect(ca_postcode_validator)

def uk_postcode_validator(sender, postcode=None, country=None, **kwargs):
    """Validates UK postcodes"""
    if country.iso2_code == 'GB':
        from l10n.validators import ukpostcode
        try:
            pc = ukpostcode.parse_uk_postcode(postcode)
        except ValueError, ve:
            log.debug('UK Postcode validator caught error: %s', ve)
            raise forms.ValidationError('Please enter a valid UK postcode.')
        return ' '.join(pc)

signals.validate_postcode.connect(uk_postcode_validator)

def us_postcode_validator(sender, postcode=None, country=None, **kwargs):
    if country.iso2_code == 'US':
        from l10n.validators import uspostcode
        try:
            pc = uspostcode.validate(postcode)
            return pc
        except ValueError, ve:
            raise forms.ValidationError('Please enter a valid US ZIP code.')
signals.validate_postcode.connect(us_postcode_validator)



from django import template
from django.utils import translation
from livesettings import config_get_group
from payment.modules.google import CHECKOUT_BUTTON_SIZES
from django.utils.http import urlencode

register = template.Library()

def _truefalse(val, t="1", f="0"):
    if val:
        return t
    else:
        return f

def checkout_image_url(merchid, imgsize, locale, transparent=False, disabled=False):
    payment_module = config_get_group('PAYMENT_GOOGLE')
    dimensions = CHECKOUT_BUTTON_SIZES[imgsize]
    return ("%s?%s" % (
        payment_module.CHECKOUT_BUTTON_URL,
        urlencode((('merchant_id', merchid),
                  ('w', dimensions[0]),
                  ('h', dimensions[1]),
                  ('style', _truefalse(transparent, t="trans", f="white")),
                  ('variant', _truefalse(disabled, t="disabled", f="text")),
                  ('loc', locale)))))


class GoogleCheckoutImageUrlNode(template.Node):
    def __init__(self, merchid, imgsize, transparent, disabled):
        self.merchid = merchid
        self.imgsize = imgsize
        self.transparent = transparent
        self.disabled = disabled

    def render(self, context):
        lang = translation.get_language()
        locale = translation.to_locale(lang)

        return checkout_image_url(self.merchid, self.imgsize, locale, transparent=self.transparent, disabled=self.disabled)


def google_checkout_image_url(parser, token):
    """
    Render the url for a google checkout image.

    Sample usage::

      {% google_checkout_image_url [imagesize] ['transparent'] ['disabled'] %}

    """
    args = token.split_contents()
    payment_module = config_get_group('PAYMENT_GOOGLE')
    merchid = payment_module.MERCHANT_ID
    sizes = CHECKOUT_BUTTON_SIZES.keys()

    imgsize = "MEDIUM"
    transparent = False
    disabled = False
    locale = None

    for arg in args[1:]:
        k = arg.upper()
        if k == 'TRANSPARENT':
            transparent = True
        elif k == 'DISABLED':
            disabled = True
        else:
            if k in sizes:
                imgsize = k
            else:
                raise template.TemplateSyntaxError("%r tag got an unexpected argument.  Perhaps a bad size?  Didn't know: %s" % (args[0], arg))
                
    return GoogleCheckoutImageUrlNode(merchid, imgsize, transparent, disabled)

register.tag(google_checkout_image_url)



from django.contrib.sites.models import SiteManager, Site
from threaded_multihost import sites

def site_get_current(self):
    """Overridden version of get_current, which is multihost aware."""
    return sites.by_host()

SiteManager.get_current = site_get_current
SiteManager.MULTIHOST = True



#!/usr/bin/env python
# This test suite verifies the environment for common misconfiguration
# problems.

import os, sys

from notmm.utils.configparse import is_string
from notmm.utils.django_compat import get_callable
from notmm.utils.wsgilib import HTTPRequest

from test_support import unittest

class SanityCheckTestCase(unittest.TestCase):

    def test_get_callable(self):
        #Convert a string version of a function name to the callable object.
        #If the lookup_view is not an import path, it is assumed to be a URL pattern
        #label and the original string is returned.
        #If can_fail is True, lookup_view might be a URL pattern label, so errors
        #during the import fail and the string is returned.
        lookup_view = get_callable('moviereviews.views.test_schevo_database')

        # lookup_view must be a callable
        self.assertEqual('__call__' in dir(lookup_view), True)
        
        # The name of the function should match the view name
        self.assertEqual(lookup_view.__name__ is 'test_schevo_database', True)

    def test_django_home(self):
        # Detection of DJANGO_HOME in $ENV. DJANGO_HOME must
        # be set and the ``django`` package must be importable
        # from this directory.

        keyname = 'DJANGO_HOME'

        have_django_home = bool(is_string(os.environ.get(keyname, '')))
        self.assertEqual(have_django_home, True)

        # if we get here then we can use a shortcut..
        django_home = os.environ[keyname]

        # Check if the django_home path exists..
        self.assertEqual(os.path.exists(django_home), True, """\
Whoa there! Your DJANGO_HOME setting is invalid. Please verify if the path exists and run the test again.
No such path: %s""" % django_home) 

        # And if the ``django`` package can be imported from that
        # specified location...
        self.assertEqual(sys.path.__contains__(django_home), True, """\
Your PYTHONPATH setting seems invalid. It should contains %s.""" % keyname)

        #mod = __import__('django', globals(), locals(), fromlist=[''])
        #basedir = os.path.realpath(getattr(mod, '__file__').split('/')[:-1])
        #self.assertEqual(basedir==DJANGO_HOME, 'basedir: %s' % basedir)
        
    def test_django_settings_module(self):
        # DJANGO_SETTINGS_MODULE must be importable
        try:
            settings_module = __import__(os.environ['DJANGO_SETTINGS_MODULE'], {}, {}, fromlist=[''])
        except ImportError:
            # this is for backward-compatibility reasons..
            self.fail("""\
%s is not doing currently anything.
Hint: Your PYTHONPATH is probably wrong and/or out-of-date.""" %
SETTINGS_MODULE_NAME)
    
    def test_django_request_context(self):
        # For this to work RequestContext must be
        # a subclass of a dictionary type !
        from django.template import RequestContext, Context
        self.failUnlessEqual(issubclass(RequestContext, Context), True) 




from django.contrib import admin
from app_plugins.models import PluginPoint, Plugin, UserPluginPreference
from app_plugins.forms import AdminPluginPointForm, AdminPluginForm

class PluginPointAdmin(admin.ModelAdmin):
    list_display = ('label', 'index', 'registered', 'status')
    list_filter = ('registered', 'status')
    form = AdminPluginPointForm

class PluginAdmin(admin.ModelAdmin):
    list_display = ('label', 'index', 'registered', 'required', 'status')
    list_filter = ('registered', 'status')
    form = AdminPluginForm

class UserPluginPreferenceAdmin(admin.ModelAdmin):
    list_display = ('plugin', 'user', 'index', 'visible')
    list_filter = ('visible',)

admin.site.register(PluginPoint, PluginPointAdmin)
admin.site.register(Plugin, PluginAdmin)
admin.site.register(UserPluginPreference, UserPluginPreferenceAdmin)




"""
Urls for wishlists, note that this does not have to get added manually to the urls, it will be added automatically by satchmo core if this app is installed.
"""
from django.conf.urls.defaults import *
from livesettings import config_value
import logging

log = logging.getLogger('wishlist.urls')

urlpatterns = patterns('satchmo_ext.wishlist.views',
     (r'^$', 'wishlist_view', {}, 'satchmo_wishlist_view'),
     (r'^add/$', 'wishlist_add', {}, 'satchmo_wishlist_add'),
     (r'^add/ajax/$', 'wishlist_add_ajax', {}, 'satchmo_wishlist_add_ajax'),
     (r'^remove/$', 'wishlist_remove', {}, 'satchmo_wishlist_remove'),
     (r'^remove/ajax$', 'wishlist_remove_ajax', {}, 'satchmo_wishlist_remove_ajax'),
     (r'^add_cart/$', 'wishlist_move_to_cart', {}, 'satchmo_wishlist_move_to_cart'),
)

wishbase = r'^' + config_value('SHOP','WISHLIST_SLUG') + '/'    
wishpatterns = patterns('',
    (wishbase, include('satchmo_ext.wishlist.urls'))
)

def add_wishlist_urls(sender, patterns=(), **kwargs):
    log.debug('adding wishlist urls at %s', wishbase)
    patterns += wishpatterns



#!/usr/bin/env
# Copyright (C) 2011-2017 Etienne Robillard <tkadm30@yandex.com>

# Provides the argparse2.ArgumentParser class with an extra
# method to load pre-defined parser options/arguments by using
# the add_argument method. 

import argparse
import yaml
import sys

class ArgumentParserError(Exception):
    pass

class ArgumentParser(argparse.ArgumentParser):

    def __init__(self, f):
        super(ArgumentParser, self).__init__(f)

    def loadArgumentsFromString(self, source, label="command_spec", 
        nargs=1, default=None, dest=None):
        """Loads a set of predefined options from a yaml stream.
        
        Options are in the format:

        command_spec:
            name: 'config spec for project bar'
            option1:    
                alternatives: --option1
            option2:
                dest: "filename",
                nargs: 2    
            option3:
                action: "store_true"
                help_text: "runs commands xyz"
                default: False
        """
        argmap = yaml.safe_load(source)
        if not label in argmap.keys():
            raise ArgumentParserError('invalid command spec!')
        else:
            for name, opts in argmap[label].iteritems():

                arglist = ["--%s"%name,]
                
                if 'alternatives' in opts:
                    alternatives = ["-%s"%item for item in opts['alternatives'].split(',')]
                
                if len(alternatives) >= 1:
                    arglist.extend(alternatives)

                if not 'action' in opts:
                    action = 'store_true'
                else:
                    action = opts.pop('action')
                    
                help_text = opts.pop('help_text')
                default = opts.pop('default')
                dest = opts.pop('dest')

                self.add_argument(arglist, action=action, \
                    help=help_text, default=default)
                    



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright 2017-2019 Etienne Robillard <tkadm30@yandex.com>

import logging
from notmm.controllers.wsgi import WSGIController
from notmm.dbapi.orm import ClientStorageProxy
logger = logging.getLogger('notmm.controllers.wsgi')

__all__ = ['ZODBController']

class ZODBController(WSGIController):

    key_prefix = 'schevo.db.'
    debug = True

    def __init__(self, request, db_name, manager=None, **kwargs):
        super(ZODBController, self).__init__(**kwargs)
        self.environ_key = self.key_prefix + 'zodb'
        self.manager = manager
        self.setup_database(db_name)
        
    def setup_database(self, db_name):
        if self.manager is None:
            #raise ValueError("Database manager is not set!")
            #Backward-compatible mode!

            self.db = ClientStorageProxy(db_name)
        else:
            try:
                #print self.manager.connections
                self.db = self.manager[db_name]
                if self.debug:
                    assert self.db is not None
            except (KeyError, AttributeError):
                raise
        if self.debug:
            logger.debug("Configured database: %s" % self.db._label)
        return None
    def init_request(self, environ):
        request = super(ZODBController, self).init_request(environ)
        if self.debug:
            assert self.environ_key == 'schevo.db.zodb' # XXX use settings.SCHEVO['DATABASE_URL']

        if not self.environ_key in request.environ:
            request.environ[self.environ_key] = self.db
        self._request = request # cheat!!!
        return request



from django.contrib.sites.models import SiteManager

def is_multihost_enabled():
    return getattr(SiteManager, 'MULTIHOST', False)

def clean_field(form, field_name):
    return form.fields[field_name].clean(form.data.get(field_name))



import os
import yaml

from notmm.utils.YAMLFixture import YAMLFixture

from test_support import unittest

class YAMLFixtureTestCase(unittest.TestCase):
    
    def setUp(self):
        self.filename = os.path.join(os.getcwd(), 'lib/site-packages/sandbox/fixtures/initial_data.yaml')
        self.fixture = YAMLFixture(self.filename, schema_index_root='packages')
    
    def tearDown(self):
        self.fixture = None

    def test_schema_index_root(self):
        # full dump of initial_data (no index_root parameter specified)
        #self.fixture = YAMLFixture(self.filename)
        da = [item for item in self.fixture]
        db = self.fixture.dumps()
        self.assertEqual(da, db)

    def test_get(self):
        d1 = self.fixture.get(name='notmm')
        d2 = self.fixture.get(name='django.bugfixes')
        d3 = self.fixture.get(name='teatime')
        
        self.assertEqual(isinstance(d1, dict), True)
        self.assertEqual('name' in d1.keys(), True)

        self.assertEqual(d1 != d2, True, repr([d1, d2]))
        self.assertEqual(d1 != d3, True)
        self.assertEqual(d2 != d3, True)

        #print d1, d2, d3

    def test_filter(self):
        l1 = self.fixture.filter(category=u'Web Programming')
        self.assertEqual(isinstance(l1, list), True)
        l2 = self.fixture.filter(name="BlogEngine")

    
    def test_dumps(self):
        sample_data = self.fixture.dumps()
        self.assertEqual(isinstance(sample_data, list), True)
        self.assertEqual(len(sample_data) == 7, True, len(sample_data))

    def test_tojson(self):
        # encode the data to json
        json = self.fixture.tojson(strict=True)
        self.assertEqual(isinstance(json, basestring), True, type(json))

    def test_pathname(self):
        pathname = self.fixture.path
        self.assertEqual(os.path.exists(pathname), True)





def _stripquotes(val):
    stripping = True
    while stripping:
        stripping = False
        if val[0] in ('"', "'"):
            val = val[1:]
            stripping = True
        if val[-1] in ('"', "'"):
            val = val[:-1]
            stripping = True

    return val

def get_filter_args(argstring, keywords=(), intargs=(), boolargs=(), stripquotes=False):
    """Convert a string formatted list of arguments into a kwargs dictionary.
    Automatically converts all keywords in intargs to integers.

    If keywords is not empty, then enforces that only those keywords are returned.
    Also handles args, which are just elements without an equal sign

    ex:
    in: get_filter_kwargs('length=10,format=medium', ('length'))
    out: (), {'length' : 10, 'format' : 'medium'}
    """
    args = []
    kwargs = {}
    if argstring:
        work = [x.strip() for x in argstring.split(',')]
        work = [x for x in work if x != '']
        for elt in work:
            parts = elt.split('=', 1)
            if len(parts) == 1:
                if stripquotes:
                    elt=_stripquotes(elt)
                args.append(elt)

            else:
                key, val = parts
                val = val.strip()
                if stripquotes and val:
                    val=_stripquotes(val)

                key = key.strip()
                if not key: continue
                key = key.lower().encode('ascii')

                if not keywords or key in keywords:
                    if key in intargs:
                        try:
                            val = int(val)
                        except ValueError:
                            raise ValueError('Could not convert value "%s" to integer for keyword "%s"' % (val, key))
                    if key in boolargs:
                        val = val.lower()
                        val = val in (1, 't', 'true', 'yes', 'y', 'on')
                    kwargs[key] = val
    return args, kwargs





from django.contrib import admin
from django.utils.translation import ugettext_lazy as _
from product.admin import ProductOptions
from product.models import Product
from satchmo_ext.tieredpricing.models import TieredPrice, PricingTier
from product.admin import Price_Inline

class TieredPriceInline(admin.TabularInline):
    model = TieredPrice
    extra = 2
    verbose_name = _("Tiered Price")
    verbose_name_plural = _("Tiered Prices")    


class PricingTierOptions(admin.ModelAdmin):
    pass

admin.site.register(PricingTier, PricingTierOptions)
admin.site.unregister(Product)
ix = 0
while ix < len(ProductOptions.inlines):
    if ProductOptions.inlines[ix] == Price_Inline:
        ProductOptions.inlines.insert(ix+1, TieredPriceInline)
    ix += 1
    
admin.site.register(Product, ProductOptions)



"""A full cache system written on top of Django's rudimentary one."""

from django.conf import settings
from django.core.cache import cache
from django.utils.encoding import smart_str
from satchmo_utils import is_string_like, is_list_or_tuple
import cPickle as pickle
import logging
import types
import hashlib

log = logging.getLogger('keyedcache')

CACHED_KEYS = {}
CACHE_CALLS = 0
CACHE_HITS = 0
KEY_DELIM = "::"
REQUEST_CACHE = {'enabled' : False}
try:
    CACHE_PREFIX = settings.CACHE_PREFIX
except AttributeError:
    CACHE_PREFIX = str(settings.SITE_ID)
    log.warn("No CACHE_PREFIX found in settings, using SITE_ID.  Please update your settings to add a CACHE_PREFIX")

try:
    CACHE_TIMEOUT = settings.CACHE_TIMEOUT
except AttributeError:
    CACHE_TIMEOUT = 0
    log.warn("No CACHE_TIMEOUT found in settings, so we used 0, disabling the cache system.  Please update your settings to add a CACHE_TIMEOUT and avoid this warning.")

_CACHE_ENABLED = CACHE_TIMEOUT > 0

class CacheWrapper(object):
    def __init__(self, val, inprocess=False):
        self.val = val
        self.inprocess = inprocess

    def __str__(self):
        return str(self.val)

    def __repr__(self):
        return repr(self.val)

    def wrap(cls, obj):
        if isinstance(obj, cls):
            return obj
        else:
            return cls(obj)

    wrap = classmethod(wrap)

class MethodNotFinishedError(Exception): 
    def __init__(self, f):
        self.func = f


class NotCachedError(Exception):    
    def __init__(self, k):
        self.key = k

class CacheNotRespondingError(Exception):    
    pass
    
def cache_delete(*keys, **kwargs):
    removed = []
    if cache_enabled():
        global CACHED_KEYS
        log.debug('cache_delete')
        children = kwargs.pop('children',False)

        if (keys or kwargs):
            key = cache_key(*keys, **kwargs)
    
            if CACHED_KEYS.has_key(key):
                del CACHED_KEYS[key]
                removed.append(key)

            cache.delete(key)

            if children:
                key = key + KEY_DELIM
                children = [x for x in CACHED_KEYS.keys() if x.startswith(key)]
                for k in children:
                    del CACHED_KEYS[k]
                    cache.delete(k)
                    removed.append(k)
        else:
            key = "All Keys"
            deleteneeded = _cache_flush_all()
        
            removed = CACHED_KEYS.keys()

            if deleteneeded:
                for k in CACHED_KEYS:
                    cache.delete(k)
            
            CACHED_KEYS = {}

        if removed:
            log.debug("Cache delete: %s", removed)
        else:
            log.debug("No cached objects to delete for %s", key)

    return removed


def cache_delete_function(func):
    return cache_delete(['func', func.__name__, func.__module__], children=True)

def cache_enabled():
    global _CACHE_ENABLED
    return _CACHE_ENABLED

def cache_enable(state=True):
    global _CACHE_ENABLED
    _CACHE_ENABLED=state

def _cache_flush_all():
    if is_memcached_backend():
        cache._cache.flush_all()
        return False
    return True

def cache_function(length=CACHE_TIMEOUT):
    """
    A variant of the snippet posted by Jeff Wheeler at
    http://www.djangosnippets.org/snippets/109/

    Caches a function, using the function and its arguments as the key, and the return
    value as the value saved. It passes all arguments on to the function, as
    it should.

    The decorator itself takes a length argument, which is the number of
    seconds the cache will keep the result around.

    It will put a temp value in the cache while the function is
    processing. This should not matter in most cases, but if the app is using
    threads, you won't be able to get the previous value, and will need to
    wait until the function finishes. If this is not desired behavior, you can
    remove the first two lines after the ``else``.
    """
    def decorator(func):
        def inner_func(*args, **kwargs):
            if not cache_enabled():
                value = func(*args, **kwargs)
                
            else:        
                try:
                    value = cache_get('func', func.__name__, func.__module__, args, kwargs)

                except NotCachedError, e:
                    # This will set a temporary value while ``func`` is being
                    # processed. When using threads, this is vital, as otherwise
                    # the function can be called several times before it finishes
                    # and is put into the cache.
                    funcwrapper = CacheWrapper(".".join([func.__module__, func.__name__]), inprocess=True)
                    cache_set(e.key, value=funcwrapper, length=length, skiplog=True)
                    value = func(*args, **kwargs)
                    cache_set(e.key, value=value, length=length)
                
                except MethodNotFinishedError, e:
                    value = func(*args, **kwargs)

            return value
        return inner_func
    return decorator


def cache_get(*keys, **kwargs):
    if kwargs.has_key('default'):
        default_value = kwargs.pop('default')
        use_default = True
    else:
        use_default = False

    key = cache_key(keys, **kwargs)
    
    if not cache_enabled():
        raise NotCachedError(key)
    else:
        global CACHE_CALLS, CACHE_HITS, REQUEST_CACHE
        CACHE_CALLS += 1
        if CACHE_CALLS == 1:
            cache_require()
        
        obj = None
        tid = -1
        if REQUEST_CACHE['enabled']:
            tid = cache_get_request_uid()
            if tid > -1:
                try:
                    obj = REQUEST_CACHE[tid][key]
                    log.debug('Got from request cache: %s', key)
                except KeyError:
                    pass

        if obj == None:
            obj = cache.get(key)
            
        if obj and isinstance(obj, CacheWrapper):
            CACHE_HITS += 1
            CACHED_KEYS[key] = True
            log.debug('got cached [%i/%i]: %s', CACHE_CALLS, CACHE_HITS, key)
            if obj.inprocess:
                raise MethodNotFinishedError(obj.val)
            
            cache_set_request(key, obj, uid=tid)
            
            return obj.val
        else:
            try:
                del CACHED_KEYS[key]
            except KeyError:
                pass

            if use_default:
                return default_value
    
            raise NotCachedError(key)


def cache_set(*keys, **kwargs):
    """Set an object into the cache."""
    if cache_enabled():
        global CACHED_KEYS, REQUEST_CACHE
        obj = kwargs.pop('value')
        length = kwargs.pop('length', CACHE_TIMEOUT)
        skiplog = kwargs.pop('skiplog', False)

        key = cache_key(keys, **kwargs)
        val = CacheWrapper.wrap(obj)
        if not skiplog:
            log.debug('setting cache: %s', key)
        cache.set(key, val, length)
        CACHED_KEYS[key] = True
        if REQUEST_CACHE['enabled']:
            cache_set_request(key, val)

def _hash_or_string(key):
    if is_string_like(key) or isinstance(key, (types.IntType, types.LongType, types.FloatType)):
        return smart_str(key)
    else:
        try:
            #if it has a PK, use it.
            return str(key._get_pk_val())
        except AttributeError:
            return md5_hash(key)

def cache_contains(*keys, **kwargs):
    key = cache_key(keys, **kwargs)
    return CACHED_KEYS.has_key(key)

def cache_key(*keys, **pairs):
    """Smart key maker, returns the object itself if a key, else a list 
    delimited by ':', automatically hashing any non-scalar objects."""

    if is_string_like(keys):
        keys = [keys]
        
    if is_list_or_tuple(keys):
        if len(keys) == 1 and is_list_or_tuple(keys[0]):
            keys = keys[0]
    else:
        keys = [md5_hash(keys)]

    if pairs:
        keys = list(keys)
        klist = pairs.keys()
        klist.sort()
        for k in klist:
            keys.append(k)
            keys.append(pairs[k])
    
    key = KEY_DELIM.join([_hash_or_string(x) for x in keys])
    prefix = CACHE_PREFIX + KEY_DELIM
    if not key.startswith(prefix):
        key = prefix+key
    return key.replace(" ", ".")

def md5_hash(obj):
    pickled = pickle.dumps(obj, protocol=pickle.HIGHEST_PROTOCOL)
    return hashlib.md5(pickled).hexdigest()


def is_memcached_backend():
    try:
        return cache._cache.__module__.endswith('memcache')
    except AttributeError:
        return False

def cache_require():
    """Error if keyedcache isn't running."""
    if cache_enabled():
        key = cache_key('require_cache')
        cache_set(key,value='1')
        v = cache_get(key, default = '0')
        if v != '1':
            raise CacheNotRespondingError()
        else:
            log.debug("Cache responding OK")
        return True

def cache_clear_request(uid):
    """Clears all locally cached elements with that uid"""
    global REQUEST_CACHE
    try:
        del REQUEST_CACHE[uid]
        log.debug('cleared request cache: %s', uid)
    except KeyError:
        pass

def cache_use_request_caching():
    global REQUEST_CACHE
    REQUEST_CACHE['enabled'] = True

def cache_get_request_uid():
    from threaded_multihost import threadlocals
    return threadlocals.get_thread_variable('request_uid', -1)
    
def cache_set_request(key, val, uid=None):
    if uid == None:
        uid = cache_get_request_uid()

    if uid>-1:
        global REQUEST_CACHE
        if not uid in REQUEST_CACHE:
            REQUEST_CACHE[uid] = {key:val}
        else:
            REQUEST_CACHE[tid][key] = val



from datetime import datetime
from decimal import Decimal
from django.test import TestCase
from models import Carrier, ShippingTier, Shipper

def make_tiers(carrier, prices, expires=None):
    for min_total, price in prices:
        t = ShippingTier(carrier=carrier, 
            min_total=Decimal("%i.00" % min_total),
            price=Decimal("%i.00" % price),
            expires=expires
        )
        t.save()
    

class TieredCarrierSimpleTest(TestCase):
    fixtures = []

    def testCreate(self):
        c = Carrier(key="test", active=True)
        c.save()
        t = ShippingTier(carrier=c, 
            min_total=Decimal("0.00"),
            price=Decimal("10.00"),
            )
        t.save()
        
        self.assertEqual(c.price(Decimal("0.00")), Decimal("10.00"))
        
        
class TieredCarrierPricingTest(TestCase):
    fixtures = []

    def setUp(self):
        self.carrier = Carrier(name="pricing", active=True)
        self.carrier.save()
        t = ShippingTier(carrier=self.carrier, 
            min_total=Decimal("0.00"),
            price=Decimal("10.00"),
            )
        t.save()
        
    def testBase(self):
        self.assertEqual(self.carrier.price(Decimal("0.00")), Decimal("10.00"))
        
    def test2Prices(self):
        t = ShippingTier(carrier=self.carrier, 
            min_total=Decimal("20.00"),
            price=Decimal("15.00"),
            )
        t.save()
        
        self.assertEqual(self.carrier.price(Decimal("0.00")), Decimal("10.00"))
        self.assertEqual(self.carrier.price(Decimal("20.00")), Decimal("15.00"))

    def test4Prices(self):
        prices = (
            (20, 15),
            (30, 16),
            (40, 17)
        )
        
        make_tiers(self.carrier, prices)

        self.assertEqual(self.carrier.price(Decimal("0.00")), Decimal("10.00"))
        self.assertEqual(self.carrier.price(Decimal("20.00")), Decimal("15.00"))
        self.assertEqual(self.carrier.price(Decimal("25.00")), Decimal("15.00"))
        self.assertEqual(self.carrier.price(Decimal("30.00")), Decimal("16.00"))
        self.assertEqual(self.carrier.price(Decimal("39.99")), Decimal("16.00"))
        self.assertEqual(self.carrier.price(Decimal("40.01")), Decimal("17.00"))


class TieredCarrierExpiringTest(TestCase):
    fixtures = []

    def setUp(self):
        self.carrier = Carrier(name="pricing", active=True)
        self.carrier.save()

        base_prices = (
            (0, 10),
            (20, 15),
            (30, 16),
            (40, 17)
        )
        make_tiers(self.carrier, base_prices)
        
    def testExpired(self):
        dt = datetime(2000, 1, 1)
        
        sale_prices = (
            (0, 1),
            (20, 2),
            (30, 3),
            (40, 4)
        )
        
        make_tiers(self.carrier, sale_prices, expires=dt)

        self.assertEqual(self.carrier.price(Decimal("0.00")), Decimal("10.00"))
        self.assertEqual(self.carrier.price(Decimal("20.00")), Decimal("15.00"))
        self.assertEqual(self.carrier.price(Decimal("30.00")), Decimal("16.00"))
        self.assertEqual(self.carrier.price(Decimal("40.01")), Decimal("17.00"))

    def testNotExpired(self):
        
        now = datetime.now()
        nextyear = datetime(now.year+1, now.month, now.day)
        sale_prices = (
            (0, 1),
            (20, 2),
            (30, 3),
            (40, 4)
        )
        
        make_tiers(self.carrier, sale_prices, expires=nextyear)

        self.assertEqual(self.carrier.price(Decimal("0.00")), Decimal("1.00"))
        self.assertEqual(self.carrier.price(Decimal("20.00")), Decimal("2.00"))
        self.assertEqual(self.carrier.price(Decimal("30.00")), Decimal("3.00"))
        self.assertEqual(self.carrier.price(Decimal("40.01")), Decimal("4.00"))

        
        
        


import shipper
from livesettings import config_choice_values

def get_methods():
    return [shipper.Shipper(service_type=value) for value in config_choice_values('shipping.modules.ups', 'UPS_SHIPPING_CHOICES')]
    